# coding: utf8
from __future__ import unicode_literals

from ...symbols import POS, PUNCT, ADJ, NUM, DET, ADV, ADP, X, VERB
from ...symbols import NOUN, PROPN, SPACE, PRON, CONJ


TAG_MAP = {
    "ADJ__Number=Sing": {POS: ADJ},
    "ADJ___": {POS: ADJ},
    "ADP__AdpType=Prep": {POS: ADP},
    "ADP__AdpType=Preppron|Gender=Fem|Number=Sing": {POS: ADP},
    "ADP__AdpType=Preppron|Gender=Masc|Number=Plur": {POS: ADP},
    "ADP__AdpType=Preppron|Gender=Masc|Number=Sing": {POS: ADP},
    "ADV__Number=Sing": {POS: ADV},
    "ADV__PunctType=Comm": {POS: ADV},
    "ADV___": {POS: ADV},
    "Adj_Adj_N_N__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_Adj_N__Degree=Pos|Number=Plur|Variant=Short": {POS: ADJ},
    "Adj_Adj_N__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_Adj__Case=Nom|Degree=Pos": {POS: ADJ},
    "Adj_Adj__Degree=Pos": {POS: ADJ},
    "Adj_Adj__Degree=Pos|Variant=Short": {POS: ADJ},
    "Adj_Adv__Degree=Pos|Variant=Short": {POS: ADJ},
    "Adj_Adv|adv|stell|onverv_deelv__Degree=Pos|Variant=Short": {POS: ADJ},
    "Adj_Art__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_Art__Degree=Pos|Number=Sing|Variant=Short": {POS: ADJ},
    "Adj_Conj_V__Degree=Pos|Mood=Sub|VerbForm=Fin": {POS: ADJ},
    "Adj_Int|attr|stell|vervneut__Case=Nom|Degree=Pos": {POS: ADJ},
    "Adj_Misc_Misc__Degree=Pos": {POS: ADJ},
    "Adj_N_Conj_N__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_N_N_N_N__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_N_N_N__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_N_N__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_N_Num__Definite=Def|Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_N_Prep_Art_Adj_N__Degree=Pos|Gender=Neut|Number=Sing": {POS: ADJ},
    "Adj_N_Prep_N_Conj_N__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_N_Prep_N_N__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_N_Prep_N__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_N_Punc__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_N__Degree=Pos|Number=Plur": {POS: ADJ},
    "Adj_N__Degree=Pos|Number=Sing": {POS: ADJ},
    "Adj_N__Degree=Pos|Number=Sing|Variant=Short": {POS: ADJ},
    "Adj_Num__Definite=Def|Degree=Pos": {POS: ADJ},
    "Adj_Num__Definite=Def|Degree=Pos|Variant=Short": {POS: ADJ},
    "Adj_Prep|adv|stell|vervneut_voor__Degree=Pos|Variant=Short": {POS: ADJ},
    "Adj_Prep|adv|vergr|onverv_voor__Degree=Cmp|Variant=Short": {POS: ADJ},
    "Adj_V_Conj_V__Degree=Pos|VerbForm=Inf": {POS: ADJ},
    "Adj_V_N__Degree=Pos|Number=Sing|Tense=Past|VerbForm=Part": {POS: ADJ},
    "Adj_V|adv|stell|onverv_intrans|inf__Degree=Pos|Variant=Short|VerbForm=Inf": {
        POS: ADJ
    },
    "Adj_V|adv|stell|onverv_trans|imp__Degree=Pos|Mood=Imp|Variant=Short|VerbForm=Fin": {
        POS: ADJ
    },
    "Adj|adv|stell|onverv__Degree=Pos|Variant=Short": {POS: ADJ},
    "Adj|adv|stell|vervneut__Case=Nom|Degree=Pos|Variant=Short": {POS: ADJ},
    "Adj|adv|vergr|onverv__Degree=Cmp|Variant=Short": {POS: ADJ},
    "Adj|adv|vergr|vervneut__Case=Nom|Degree=Cmp|Variant=Short": {POS: ADJ},
    "Adj|attr|overtr|onverv__Degree=Sup": {POS: ADJ},
    "Adj|attr|overtr|vervneut__Case=Nom|Degree=Sup": {POS: ADJ},
    "Adj|attr|stell|onverv__Degree=Pos": {POS: ADJ},
    "Adj|attr|stell|vervgen__Case=Gen|Degree=Pos": {POS: ADJ},
    "Adj|attr|stell|vervneut__Case=Nom|Degree=Pos": {POS: ADJ},
    "Adj|attr|vergr|onverv__Degree=Cmp": {POS: ADJ},
    "Adj|attr|vergr|vervgen__Case=Gen|Degree=Cmp": {POS: ADJ},
    "Adj|attr|vergr|vervneut__Case=Nom|Degree=Cmp": {POS: ADJ},
    "Adj|zelfst|overtr|vervneut__Case=Nom|Degree=Sup": {POS: ADJ},
    "Adj|zelfst|stell|onverv__Degree=Pos": {POS: ADJ},
    "Adj|zelfst|stell|vervmv__Degree=Pos|Number=Plur": {POS: ADJ},
    "Adj|zelfst|stell|vervneut__Case=Nom|Degree=Pos": {POS: ADJ},
    "Adj|zelfst|vergr|vervneut__Case=Nom|Degree=Cmp": {POS: ADJ},
    "Adv_Adj_Conj__Degree=Pos": {POS: ADV},
    "Adv_Adj__Degree=Cmp": {POS: ADV},
    "Adv_Adj__Degree=Pos": {POS: ADV},
    "Adv_Adv_Conj_Adv__PronType=Dem": {POS: ADV},
    "Adv_Adv__AdpType=Prep": {POS: ADV},
    "Adv_Adv__Degree=Pos": {POS: ADV},
    "Adv_Adv__Degree=Pos|PronType=Dem": {POS: ADV},
    "Adv_Adv|pron|vrag_deeladv___": {POS: ADV},
    "Adv_Art__Degree=Pos|Number=Sing": {POS: ADV},
    "Adv_Art__Number=Sing": {POS: ADV},
    "Adv_Conj_Adv__AdpType=Preppron|Gender=Masc|Number=Sing": {POS: ADV},
    "Adv_Conj_Adv__Degree=Pos": {POS: ADV},
    "Adv_Conj_Adv|gew|aanw_neven_gew|aanw__PronType=Dem": {POS: ADV},
    "Adv_Conj_Adv|gew|onbep_neven_gew|onbep__PronType=Ind": {POS: ADV},
    "Adv_Conj_N__Degree=Pos|Number=Sing": {POS: ADV},
    "Adv_Conj__Degree=Pos": {POS: ADV},
    "Adv_N__Degree=Pos|Number=Sing": {POS: ADV},
    "Adv_Num__Degree=Cmp|PronType=Ind": {POS: ADV},
    "Adv_N|gew|aanw_soort|ev|neut__Number=Sing": {POS: ADV},
    "Adv_Prep_N__Case=Dat|Degree=Pos|Number=Sing": {POS: ADV},
    "Adv_Prep_Pron__AdpType=Preppron|Gender=Masc|Number=Sing": {POS: ADV},
    "Adv_Prep__Degree=Pos": {POS: ADV},
    "Adv_Prep|gew|aanw_voor__AdpType=Prep": {POS: ADV},
    "Adv_Prep|gew|aanw_voor___": {POS: ADV},
    "Adv_Pron__Degree=Pos": {POS: ADV},
    "Adv|deeladv__PartType=Vbp": {POS: ADV},
    "Adv|deelv__PartType=Vbp": {POS: ADV},
    "Adv|gew|aanw__PronType=Dem": {POS: ADV},
    "Adv|gew|betr__PronType=Rel": {POS: ADV},
    "Adv|gew|er__AdvType=Ex": {POS: ADV},
    "Adv|gew|geenfunc|overtr|onverv__Degree=Sup": {POS: ADV},
    "Adv|gew|geenfunc|stell|onverv__Degree=Pos": {POS: ADV},
    "Adv|gew|geenfunc|vergr|onverv__Degree=Cmp": {POS: ADV},
    "Adv|gew|onbep__PronType=Ind": {POS: ADV},
    "Adv|gew|vrag__PronType=Int": {POS: ADV},
    "Adv|pron|aanw__PronType=Dem": {POS: ADV},
    "Adv|pron|betr__PronType=Rel": {POS: ADV},
    "Adv|pron|er__AdvType=Ex": {POS: ADV},
    "Adv|pron|onbep__PronType=Ind": {POS: ADV},
    "Adv|pron|vrag__PronType=Int": {POS: ADV},
    "Art_Adj_N__AdpType=Prep": {POS: DET},
    "Art_Adj_N__Definite=Def|Degree=Sup|Gender=Neut|Number=Sing": {POS: DET},
    "Art_Adj__Case=Nom|Definite=Def|Degree=Cmp|Gender=Neut": {POS: DET},
    "Art_Adj__Case=Nom|Definite=Def|Degree=Sup|Gender=Neut": {POS: DET},
    "Art_Adj__Definite=Def|Degree=Cmp|Gender=Neut": {POS: DET},
    "Art_Adj__Definite=Def|Degree=Sup|Gender=Neut": {POS: DET},
    "Art_Adv__Definite=Def|Degree=Sup|Gender=Neut": {POS: DET},
    "Art_Conj_Pron__Number=Sing|PronType=Ind": {POS: DET},
    "Art_N_Conj_Art_N__Definite=Def|Gender=Neut|Number=Sing": {POS: DET},
    "Art_N_Conj_Art_V__AdpType=Prep": {POS: DET},
    "Art_N_Conj_Pron_N__Definite=Def|Gender=Neut|Number=Plur|Person=3": {POS: DET},
    "Art_N_Conj__Number=Sing|PronType=Ind": {POS: DET},
    "Art_N_N__AdpType=Prep": {POS: DET},
    "Art_N_Prep_Adj__Degree=Pos|Number=Sing|PronType=Ind": {POS: DET},
    "Art_N_Prep_Art_N__Number=Sing|PronType=Ind": {POS: DET},
    "Art_N_Prep_N__AdpType=Prep": {POS: DET},
    "Art_N_Prep_N__Definite=Def|Gender=Neut|Number=Sing": {POS: DET},
    "Art_N_Prep_N__Number=Sing|PronType=Ind": {POS: DET},
    "Art_N_Prep_Pron_N__AdpType=Prep": {POS: DET},
    "Art_N__AdpType=Prep": {POS: DET},
    "Art_N__Case=Gen|Definite=Def|Number=Sing": {POS: DET},
    "Art_N__Number=Sing|PronType=Ind": {POS: DET},
    "Art_Num_Art_Adj__AdpType=Prep": {POS: DET},
    "Art_Num_N__AdpType=Prep": {POS: DET},
    "Art_Num__Definite=Def|Degree=Sup|Gender=Neut|PronType=Ind": {POS: DET},
    "Art_Num__Definite=Def|Gender=Neut": {POS: DET},
    "Art_Num__Degree=Pos|Number=Sing|PronType=Ind": {POS: DET},
    "Art_N|bep|onzijd|neut_eigen|ev|neut__Definite=Def|Gender=Neut|Number=Sing": {
        POS: DET
    },
    "Art_N|bep|onzijd|neut_soort|ev|neut__Definite=Def|Gender=Neut|Number=Sing": {
        POS: DET
    },
    "Art_Pron_N__Case=Gen|Number=Plur|PronType=Ind": {POS: DET},
    "Art_Pron__Number=Sing|PronType=Ind": {POS: DET},
    "Art_V_N__AdpType=Prep": {POS: DET},
    "Art|bep|onzijd|neut__Definite=Def|Gender=Neut|PronType=Art": {POS: DET},
    "Art|bep|zijdofmv|gen__Case=Gen|Definite=Def|PronType=Art": {POS: DET},
    "Art|bep|zijdofmv|neut__Definite=Def|PronType=Art": {POS: DET},
    "Art|bep|zijdofonzijd|gen__Case=Gen|Definite=Def|Number=Sing|PronType=Art": {
        POS: DET
    },
    "Art|bep|zijd|dat__Case=Dat|Definite=Def|Gender=Com|PronType=Art": {POS: DET},
    "Art|onbep|zijdofonzijd|neut__Definite=Ind|Number=Sing|PronType=Art": {POS: DET},
    "CCONJ___": {POS: CONJ},
    "Conj_Adj|neven_adv|vergr|onverv__Degree=Cmp": {POS: CONJ},
    "Conj_Adj|neven_attr|stell|onverv__Degree=Pos": {POS: CONJ},
    "Conj_Adv_Adv__Degree=Pos": {POS: CONJ},
    "Conj_Adv__AdpType=Prep": {POS: CONJ},
    "Conj_Adv__AdpType=Preppron|Gender=Masc|Number=Plur": {POS: CONJ},
    "Conj_Adv__Degree=Pos": {POS: CONJ},
    "Conj_Adv|neven_gew|aanw__PronType=Dem": {POS: CONJ},
    "Conj_Art_N__AdpType=Preppron|Gender=Masc|Number=Plur": {POS: CONJ},
    "Conj_Art_N__Gender=Neut|Number=Sing": {POS: CONJ},
    "Conj_Conj|neven_onder|metfin___": {POS: CONJ},
    "Conj_Int|neven___": {POS: CONJ},
    "Conj_Int|onder|metfin___": {POS: CONJ},
    "Conj_N_Adv__AdpType=Preppron|Gender=Masc|Number=Plur": {POS: CONJ},
    "Conj_N_Prep__AdpType=Preppron|Gender=Masc|Number=Plur": {POS: CONJ},
    "Conj_N|onder|metfin_soort|ev|neut__AdpType=Preppron|Gender=Masc|Number=Plur": {
        POS: CONJ
    },
    "Conj_Pron_Adv__Degree=Pos|Number=Sing|Person=3": {POS: CONJ},
    "Conj_Pron_V__AdpType=Preppron|Gender=Masc|Number=Plur": {POS: CONJ},
    "Conj_Pron|neven_aanw|neut|zelfst__AdpType=Prep": {POS: CONJ},
    "Conj_Punc_Conj|neven_schuinstreep_neven__AdpType=Prep": {POS: CONJ},
    "Conj_V|onder|metfin_intrans|ott|3|ev__AdpType=Preppron|Gender=Masc|Number=Plur": {
        POS: CONJ
    },
    "Conj|neven___": {POS: CONJ},
    "Conj|onder|metfin___": {POS: CONJ},
    "Conj|onder|metinf___": {POS: CONJ},
    "DET__Degree=Cmp|NumType=Card|PronType=Ind": {POS: DET},
    "DET__Gender=Fem|Number=Sing|Number[psor]=Plur|Person=1|Poss=Yes|PronType=Prs": {
        POS: DET
    },
    "DET__Gender=Fem|Number=Sing|PronType=Art": {POS: DET},
    "DET__Gender=Masc|Number=Plur|PronType=Art": {POS: DET},
    "DET__Gender=Masc|Number=Sing|PronType=Tot": {POS: DET},
    "Int_Adv|gew|aanw___": {POS: X},
    "Int_Int__NumType=Card": {POS: X},
    "Int_Int___": {POS: X},
    "Int_N_N_Misc_N___": {POS: X},
    "Int_N_Punc_Int_N__Number=Sing": {POS: X},
    "Int_Punc_Int|komma__PunctType=Comm": {POS: X},
    "Int___": {POS: X},
    "Misc_Misc_Misc_Misc_Misc_Misc_Misc_Misc_Misc___": {POS: X},
    "Misc_Misc_Misc_Misc_Misc_Misc_Misc___": {POS: X},
    "Misc_Misc_Misc_Misc_Misc_Misc_Punc_Misc_Misc_Misc___": {POS: X},
    "Misc_Misc_Misc_Misc_Misc_Misc___": {POS: X},
    "Misc_Misc_Misc_Misc_Misc_N_Misc_Misc_Misc_Misc_Misc_Misc___": {POS: X},
    "Misc_Misc_Misc_Misc|vreemd_vreemd_vreemd_vreemd__AdpType=Preppron|Gender=Masc|Number=Sing": {
        POS: X
    },
    "Misc_Misc_Misc_Misc|vreemd_vreemd_vreemd_vreemd___": {POS: X},
    "Misc_Misc_Misc_N__Number=Sing": {POS: X},
    "Misc_Misc_Misc|vreemd_vreemd_vreemd___": {POS: X},
    "Misc_Misc_N_N__Number=Sing": {POS: X},
    "Misc_Misc_N|vreemd_vreemd_soort|mv|neut__Number=Plur": {POS: X},
    "Misc_Misc_Punc_N_N__Number=Sing": {POS: X},
    "Misc_Misc|vreemd_vreemd__AdpType=Prep": {POS: X},
    "Misc_Misc|vreemd_vreemd__NumType=Card": {POS: X},
    "Misc_Misc|vreemd_vreemd___": {POS: X},
    "Misc_N_Misc_Misc__Number=Sing": {POS: X},
    "Misc_N_N__Number=Sing": {POS: X},
    "Misc_N|vreemd_eigen|ev|neut__Number=Sing": {POS: X},
    "Misc_N|vreemd_soort|ev|neut__Number=Sing": {POS: X},
    "Misc|vreemd__Foreign=Yes": {POS: X},
    "NUM__Case=Nom|Definite=Def|Degree=Pos|NumType=Card": {POS: NUM},
    "NUM__Definite=Def|Degree=Pos|NumType=Card": {POS: NUM},
    "NUM__Definite=Def|Degree=Pos|Number=Sing|NumType=Card": {POS: NUM},
    "NUM__Definite=Def|NumType=Card": {POS: NUM},
    "NUM__Definite=Def|Number=Plur|NumType=Card": {POS: NUM},
    "NUM__Definite=Def|Number=Sing|NumType=Card": {POS: NUM},
    "NUM__NumForm=Digit|NumType=Card": {POS: NUM},
    "NUM__NumType=Card": {POS: NUM},
    "N_Adj_N_Num__Definite=Def|Degree=Pos|Number=Sing": {POS: NOUN},
    "N_Adj_N__Degree=Pos|Number=Plur": {POS: NOUN},
    "N_Adj_N___": {POS: NOUN},
    "N_Adj__AdpType=Prep": {POS: NOUN},
    "N_Adj__Case=Nom|Degree=Pos|Number=Plur": {POS: NOUN},
    "N_Adj__Case=Nom|Degree=Pos|Number=Sing": {POS: NOUN},
    "N_Adj__Degree=Pos|Number=Plur": {POS: NOUN},
    "N_Adj__Degree=Pos|Number=Sing": {POS: NOUN},
    "N_Adj___": {POS: NOUN},
    "N_Adv_Punc_V_Pron_V__Aspect=Imp|Degree=Pos|Mood=Ind|Number=Sing|Person=2|Tense=Pres|VerbForm=Inf": {
        POS: NOUN
    },
    "N_Adv__Degree=Pos|Number=Sing": {POS: NOUN},
    "N_Adv___": {POS: NOUN},
    "N_Adv|soort|ev|neut_deelv__Number=Sing": {POS: NOUN},
    "N_Art_Adj_Prep_N___": {POS: NOUN},
    "N_Art_N__Case=Gen|Number=Sing": {POS: NOUN},
    "N_Art_N__Number=Plur": {POS: NOUN},
    "N_Art_N__Number=Sing": {POS: NOUN},
    "N_Art_N___": {POS: NOUN},
    "N_Conj_Adv__Degree=Pos|Number=Sing": {POS: NOUN},
    "N_Conj_Art_N___": {POS: NOUN},
    "N_Conj_N_N__Number=Sing": {POS: NOUN},
    "N_Conj_N_N___": {POS: NOUN},
    "N_Conj_N__Number=Plur": {POS: NOUN},
    "N_Conj_N__Number=Sing": {POS: NOUN},
    "N_Conj_N___": {POS: NOUN},
    "N_Conj|soort|ev|neut_neven__Number=Sing": {POS: NOUN},
    "N_Int_N|eigen|ev|neut_eigen|ev|neut___": {POS: NOUN},
    "N_Misc_Misc_Misc_Misc___": {POS: NOUN},
    "N_Misc_Misc_N___": {POS: NOUN},
    "N_Misc_Misc|eigen|ev|neut_vreemd_vreemd___": {POS: NOUN},
    "N_Misc_Misc|soort|mv|neut_vreemd_vreemd__Number=Plur": {POS: NOUN},
    "N_Misc_N_N_N_N___": {POS: NOUN},
    "N_Misc_N_N___": {POS: NOUN},
    "N_Misc_N___": {POS: NOUN},
    "N_Misc_Num___": {POS: NOUN},
    "N_Misc|eigen|ev|neut_vreemd___": {POS: NOUN},
    "N_Misc|soort|ev|neut_vreemd__Number=Sing": {POS: NOUN},
    "N_N_Adj_Art_N_N__Gender=Masc|Number=Plur|PronType=Art": {POS: NOUN},
    "N_N_Adj_N___": {POS: NOUN},
    "N_N_Adj__Degree=Pos|Number=Sing": {POS: NOUN},
    "N_N_Adj___": {POS: NOUN},
    "N_N_Art_Adv___": {POS: NOUN},
    "N_N_Art_N___": {POS: NOUN},
    "N_N_Conj_N_N_N_N_N___": {POS: NOUN},
    "N_N_Conj_N_N___": {POS: NOUN},
    "N_N_Conj_N__Number=Sing": {POS: NOUN},
    "N_N_Conj_N___": {POS: NOUN},
    "N_N_Conj___": {POS: NOUN},
    "N_N_Int_N_N___": {POS: NOUN},
    "N_N_Misc___": {POS: NOUN},
    "N_N_N_Adj_N___": {POS: NOUN},
    "N_N_N_Adv___": {POS: NOUN},
    "N_N_N_Int__AdpType=Prep": {POS: NOUN},
    "N_N_N_Misc___": {POS: NOUN},
    "N_N_N_N_Conj_N___": {POS: NOUN},
    "N_N_N_N_Misc___": {POS: NOUN},
    "N_N_N_N_N_N_Int__AdpType=Prep": {POS: NOUN},
    "N_N_N_N_N_N_N__AdpType=Prep": {POS: NOUN},
    "N_N_N_N_N_N_N__Gender=Fem|Number=Sing|PronType=Art": {POS: NOUN},
    "N_N_N_N_N_N_N___": {POS: NOUN},
    "N_N_N_N_N_N_Prep_N___": {POS: NOUN},
    "N_N_N_N_N_N__AdpType=Prep": {POS: NOUN},
    "N_N_N_N_N_N___": {POS: NOUN},
    "N_N_N_N_N_Prep_N___": {POS: NOUN},
    "N_N_N_N_N__AdpType=Prep": {POS: NOUN},
    "N_N_N_N_N__Number=Sing": {POS: NOUN},
    "N_N_N_N_N___": {POS: NOUN},
    "N_N_N_N_Prep_N___": {POS: NOUN},
    "N_N_N_N_Punc_N_Punc___": {POS: NOUN},
    "N_N_N_N_V___": {POS: NOUN},
    "N_N_N_N__Gender=Fem|Number=Plur|PronType=Art": {POS: NOUN},
    "N_N_N_N__Gender=Fem|Number=Sing|PronType=Art": {POS: NOUN},
    "N_N_N_N__NumType=Card": {POS: NOUN},
    "N_N_N_N__Number=Plur": {POS: NOUN},
    "N_N_N_N__Number=Sing": {POS: NOUN},
    "N_N_N_N___": {POS: NOUN},
    "N_N_N_Prep_Art_Adj_N___": {POS: NOUN},
    "N_N_N_Prep_N_N___": {POS: NOUN},
    "N_N_N_Prep_N___": {POS: NOUN},
    "N_N_N_Punc_N___": {POS: NOUN},
    "N_N_N_Punc___": {POS: NOUN},
    "N_N_N__AdpType=Prep": {POS: NOUN},
    "N_N_N__Gender=Fem|Number=Sing|PronType=Art": {POS: NOUN},
    "N_N_N__Gender=Masc|Number=Plur|PronType=Art": {POS: NOUN},
    "N_N_N__Number=Plur": {POS: NOUN},
    "N_N_N__Number=Sing": {POS: NOUN},
    "N_N_N___": {POS: NOUN},
    "N_N_Num_N___": {POS: NOUN},
    "N_N_Num__Definite=Def|Number=Sing": {POS: NOUN},
    "N_N_Num___": {POS: NOUN},
    "N_N_Prep_Art_Adj_N__Degree=Pos|Gender=Neut|Number=Sing": {POS: NOUN},
    "N_N_Prep_Art_N_Prep_Art_N___": {POS: NOUN},
    "N_N_Prep_Art_N___": {POS: NOUN},
    "N_N_Prep_N_N__AdpType=Prep": {POS: NOUN},
    "N_N_Prep_N_Prep_Adj_N___": {POS: NOUN},
    "N_N_Prep_N__AdpType=Prep": {POS: NOUN},
    "N_N_Prep_N__Number=Sing": {POS: NOUN},
    "N_N_Prep_N___": {POS: NOUN},
    "N_N_Punc_N_Punc___": {POS: NOUN},
    "N_Num_N_N__Definite=Def|Number=Sing": {POS: NOUN},
    "N_Num_N_Num___": {POS: NOUN},
    "N_Num_N___": {POS: NOUN},
    "N_Num_Num__Definite=Def|Number=Sing": {POS: NOUN},
    "N_Num__Definite=Def|Number=Plur": {POS: NOUN},
    "N_Num__Definite=Def|Number=Sing": {POS: NOUN},
    "N_Num___": {POS: NOUN},
    "N_N|eigen|ev|gen_eigen|ev|gen___": {POS: NOUN},
    "N_N|eigen|ev|gen_eigen|ev|neut___": {POS: NOUN},
    "N_N|eigen|ev|gen_soort|ev|neut___": {POS: NOUN},
    "N_N|eigen|ev|gen_soort|mv|neut___": {POS: NOUN},
    "N_N|eigen|ev|neut_eigen|ev|gen___": {POS: NOUN},
    "N_N|eigen|ev|neut_eigen|ev|neut__AdpType=Prep": {POS: NOUN},
    "N_N|eigen|ev|neut_eigen|ev|neut__AdpType=Preppron|Gender=Fem|Number=Plur": {
        POS: NOUN
    },
    "N_N|eigen|ev|neut_eigen|ev|neut__AdpType=Preppron|Gender=Masc|Number=Sing": {
        POS: NOUN
    },
    "N_N|eigen|ev|neut_eigen|ev|neut__Gender=Fem|Number=Plur|PronType=Art": {POS: NOUN},
    "N_N|eigen|ev|neut_eigen|ev|neut__Gender=Fem|Number=Sing|PronType=Art": {POS: NOUN},
    "N_N|eigen|ev|neut_eigen|ev|neut__Gender=Masc|Number=Plur|PronType=Art": {
        POS: NOUN
    },
    "N_N|eigen|ev|neut_eigen|ev|neut__Gender=Masc|Number=Sing|PronType=Art": {
        POS: NOUN
    },
    "N_N|eigen|ev|neut_eigen|ev|neut__NumType=Card": {POS: NOUN},
    "N_N|eigen|ev|neut_eigen|ev|neut__Number=Sing": {POS: NOUN},
    "N_N|eigen|ev|neut_eigen|ev|neut___": {POS: NOUN},
    "N_N|eigen|ev|neut_eigen|mv|neut___": {POS: NOUN},
    "N_N|eigen|ev|neut_soort|ev|neut__AdpType=Prep": {POS: NOUN},
    "N_N|eigen|ev|neut_soort|ev|neut___": {POS: NOUN},
    "N_N|eigen|ev|neut_soort|mv|neut___": {POS: NOUN},
    "N_N|eigen|mv|neut_eigen|mv|neut___": {POS: NOUN},
    "N_N|soort|ev|neut_eigen|ev|neut__Number=Sing": {POS: NOUN},
    "N_N|soort|ev|neut_soort|ev|neut__Gender=Masc|Number=Plur|PronType=Art": {
        POS: NOUN
    },
    "N_N|soort|ev|neut_soort|ev|neut__NumForm=Digit|NumType=Card": {POS: NOUN},
    "N_N|soort|ev|neut_soort|ev|neut__Number=Sing": {POS: NOUN},
    "N_N|soort|ev|neut_soort|mv|neut__Number=Plur": {POS: NOUN},
    "N_N|soort|mv|neut_eigen|ev|neut__Number=Sing": {POS: NOUN},
    "N_N|soort|mv|neut_soort|ev|neut__Number=Sing": {POS: NOUN},
    "N_N|soort|mv|neut_soort|mv|neut__Number=Plur": {POS: NOUN},
    "N_Prep_Adj_Adj_N__Degree=Pos|Number=Plur": {POS: NOUN},
    "N_Prep_Adj_N___": {POS: NOUN},
    "N_Prep_Art_N_Art_N__Number=Plur": {POS: NOUN},
    "N_Prep_Art_N_N__Number=Sing": {POS: NOUN},
    "N_Prep_Art_N_Prep_Art_N__Gender=Neut|Number=Sing": {POS: NOUN},
    "N_Prep_Art_N__Number=Plur": {POS: NOUN},
    "N_Prep_Art_N__Number=Sing": {POS: NOUN},
    "N_Prep_Art_N___": {POS: NOUN},
    "N_Prep_N_Art_Adj___": {POS: NOUN},
    "N_Prep_N_N__Number=Sing": {POS: NOUN},
    "N_Prep_N_N___": {POS: NOUN},
    "N_Prep_N_Prep_Art_N___": {POS: NOUN},
    "N_Prep_N_Prep_N_Conj_N_Prep_Art_N_N__Number=Sing": {POS: NOUN},
    "N_Prep_N_Punc_N_Conj_N__Number=Sing": {POS: NOUN},
    "N_Prep_N__Number=Plur": {POS: NOUN},
    "N_Prep_N__Number=Sing": {POS: NOUN},
    "N_Prep_N___": {POS: NOUN},
    "N_Prep_Num__Definite=Def|Number=Sing": {POS: NOUN},
    "N_Prep_Pron_N___": {POS: NOUN},
    "N_Prep|soort|ev|neut_voor__Number=Sing": {POS: NOUN},
    "N_Pron___": {POS: NOUN},
    "N_Punc_Adj_N___": {POS: NOUN},
    "N_Punc_Adj_Pron_Punc__Degree=Pos|Number=Sing|Person=2": {POS: NOUN},
    "N_Punc_Adv_V_Pron_N__Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {
        POS: NOUN
    },
    "N_Punc_Misc_Punc_N___": {POS: NOUN},
    "N_Punc_N_N_N_N__Number=Sing": {POS: NOUN},
    "N_Punc_N_Punc_N__Number=Sing": {POS: NOUN},
    "N_Punc_N_Punc__Number=Sing": {POS: NOUN},
    "N_Punc_N__Number=Sing": {POS: NOUN},
    "N_Punc_Punc_N_N_Punc_Punc_N___": {POS: NOUN},
    "N_V_N_N___": {POS: NOUN},
    "N_V_N___": {POS: NOUN},
    "N_V__Aspect=Imp|Mood=Ind|Number=Sing|Tense=Past|VerbForm=Fin": {POS: NOUN},
    "N_V__Number=Sing|Tense=Past|VerbForm=Part": {POS: NOUN},
    "N_V___": {POS: NOUN},
    "N_V|eigen|ev|neut_trans|imp___": {POS: NOUN},
    "N_V|soort|ev|neut_hulpofkopp|conj__Mood=Sub|Number=Sing|VerbForm=Fin": {POS: NOUN},
    "N_V|soort|ev|neut_intrans|conj__Mood=Sub|Number=Sing|VerbForm=Fin": {POS: NOUN},
    "Num_Adj_Adj_N___": {POS: NUM},
    "Num_Adj_N___": {POS: NUM},
    "Num_Adj__Definite=Def|Degree=Pos|NumType=Card": {POS: NUM},
    "Num_Adj__NumForm=Digit|NumType=Card": {POS: NUM},
    "Num_Adj___": {POS: NUM},
    "Num_Conj_Adj__Case=Nom|Definite=Def|Degree=Pos|NumType=Card": {POS: NUM},
    "Num_Conj_Art_Adj__Definite=Def|Degree=Pos|Number=Sing|NumType=Card": {POS: NUM},
    "Num_Conj_Num_N__NumForm=Digit|NumType=Card": {POS: NUM},
    "Num_Conj_Num__Degree=Cmp|NumType=Card|PronType=Ind": {POS: NUM},
    "Num_N_N__Definite=Def|Number=Sing|NumType=Card": {POS: NUM},
    "Num_N_Num_Num_N__NumForm=Digit|NumType=Card": {POS: NUM},
    "Num_N_Num__Definite=Def|Number=Sing|NumType=Card": {POS: NUM},
    "Num_N_Num__NumForm=Digit|NumType=Card": {POS: NUM},
    "Num_N__Definite=Def|Number=Plur|NumType=Card": {POS: NUM},
    "Num_N__Definite=Def|Number=Sing|NumType=Card": {POS: NUM},
    "Num_N__NumForm=Digit|NumType=Card": {POS: NUM},
    "Num_N___": {POS: NUM},
    "Num_Num_N__NumForm=Digit|NumType=Card": {POS: NUM},
    "Num_Num__Definite=Def|NumType=Card": {POS: NUM},
    "Num_Num__NumForm=Digit|NumType=Card": {POS: NUM},
    "Num_Prep_Num__Definite=Def|NumType=Card": {POS: NUM},
    "Num_Punc_Num_N_N__NumForm=Digit|NumType=Card": {POS: NUM},
    "Num_Punc_Num__NumForm=Digit|NumType=Card": {POS: NUM},
    "Num_Punc__NumForm=Digit|NumType=Card": {POS: NUM},
    "Num__Case=Nom|Degree=Cmp|NumType=Card|PronType=Ind": {POS: NUM},
    "Num__Case=Nom|Degree=Pos|NumType=Card|PronType=Ind": {POS: NUM},
    "Num__Case=Nom|Degree=Sup|NumType=Card|PronType=Ind": {POS: NUM},
    "Num__Degree=Cmp|NumType=Card|PronType=Ind": {POS: NUM},
    "Num__Degree=Pos|NumType=Card|PronType=Ind": {POS: NUM},
    "Num__Degree=Pos|Number=Plur|NumType=Card|PronType=Ind": {POS: NUM},
    "Num__Degree=Sup|NumType=Card|PronType=Ind": {POS: NUM},
    "Num__Degree=Sup|Number=Plur|NumType=Card|PronType=Ind": {POS: NUM},
    "Num|hoofd|bep|attr|onverv__Definite=Def|NumType=Card": {POS: NUM},
    "Num|hoofd|bep|zelfst|onverv__Definite=Def|NumType=Card": {POS: NUM},
    "Num|hoofd|bep|zelfst|vervmv__Definite=Def|Number=Plur|NumType=Card": {POS: NUM},
    "Num|hoofd|onbep|attr|stell|onverv__Degree=Pos|NumType=Card|PronType=Ind": {
        POS: NUM
    },
    "Num|hoofd|onbep|attr|vergr|onverv__Degree=Cmp|NumType=Card|PronType=Ind": {
        POS: NUM
    },
    "Num|rang|bep|attr|onverv__Definite=Def|NumType=Ord": {POS: NUM},
    "Num|rang|bep|zelfst|onverv__Definite=Def|NumType=Ord": {POS: NUM},
    "N|eigen|ev|gen__Case=Gen|Number=Sing": {POS: NOUN},
    "N|eigen|ev|neut__Number=Sing": {POS: NOUN},
    "N|eigen|mv|neut__Number=Plur": {POS: NOUN},
    "N|soort|ev|dat__Case=Dat|Number=Sing": {POS: NOUN},
    "N|soort|ev|gen__Case=Gen|Number=Sing": {POS: NOUN},
    "N|soort|ev|neut__Number=Sing": {POS: NOUN},
    "N|soort|mv|neut__Number=Plur": {POS: NOUN},
    "PROPN___": {POS: PROPN},
    "PUNCT___": {POS: PUNCT},
    "Prep_Adj_Conj_Prep_N__Degree=Pos|Number=Sing": {POS: ADP},
    "Prep_Adj_N__Degree=Pos|Number=Plur": {POS: ADP},
    "Prep_Adj|voor_adv|vergr|vervneut__Case=Nom|Degree=Cmp": {POS: ADP},
    "Prep_Adj|voor_attr|stell|onverv__Degree=Pos": {POS: ADP},
    "Prep_Adj|voor_attr|stell|vervneut__Case=Nom|Degree=Pos": {POS: ADP},
    "Prep_Adv__AdpType=Prep": {POS: ADP},
    "Prep_Adv__Case=Nom|Degree=Pos": {POS: ADP},
    "Prep_Adv__Case=Nom|Degree=Sup": {POS: ADP},
    "Prep_Adv__Degree=Pos": {POS: ADP},
    "Prep_Adv|voor_gew|aanw__AdpType=Prep": {POS: ADP},
    "Prep_Adv|voor_gew|aanw__Gender=Masc|Number=Sing|PronType=Tot": {POS: ADP},
    "Prep_Adv|voor_gew|aanw__PronType=Dem": {POS: ADP},
    "Prep_Adv|voor_pron|vrag__PronType=Int": {POS: ADP},
    "Prep_Art_Adj_N__Degree=Pos|Number=Sing": {POS: ADP},
    "Prep_Art_Adj__AdpType=Prep": {POS: ADP},
    "Prep_Art_Adj__Case=Nom|Degree=Pos": {POS: ADP},
    "Prep_Art_Adj__Degree=Cmp|Gender=Neut": {POS: ADP},
    "Prep_Art_Misc_Misc___": {POS: ADP},
    "Prep_Art_N_Adv__Number=Sing": {POS: ADP},
    "Prep_Art_N_Adv__Number=Sing|PronType=Int": {POS: ADP},
    "Prep_Art_N_Art_N__AdpType=Prep": {POS: ADP},
    "Prep_Art_N_Prep_Art_N__AdpType=Prep": {POS: ADP},
    "Prep_Art_N_Prep__AdpType=Prep": {POS: ADP},
    "Prep_Art_N_Prep__Gender=Neut|Number=Sing": {POS: ADP},
    "Prep_Art_N_Prep__Number=Sing": {POS: ADP},
    "Prep_Art_N_V__Number=Plur|Tense=Past|VerbForm=Part": {POS: ADP},
    "Prep_Art_N__AdpType=Prep": {POS: ADP},
    "Prep_Art_N__Gender=Com|Number=Sing": {POS: ADP},
    "Prep_Art_N__Gender=Neut|Number=Sing": {POS: ADP},
    "Prep_Art_N__Number=Plur": {POS: ADP},
    "Prep_Art_N__Number=Sing": {POS: ADP},
    "Prep_Art_V__AdpType=Prep": {POS: ADP},
    "Prep_Art_V__Gender=Neut|VerbForm=Inf": {POS: ADP},
    "Prep_Art|voor_bep|onzijd|neut__Gender=Neut": {POS: ADP},
    "Prep_Art|voor_onbep|zijdofonzijd|neut__Number=Sing": {POS: ADP},
    "Prep_Conj_Prep|voor_neven_voor__Gender=Masc|Number=Sing|PronType=Tot": {POS: ADP},
    "Prep_Misc|voor_vreemd___": {POS: ADP},
    "Prep_N_Adv|voor_soort|ev|neut_deeladv__Number=Sing": {POS: ADP},
    "Prep_N_Adv|voor_soort|ev|neut_pron|aanw__AdpType=Prep": {POS: ADP},
    "Prep_N_Adv|voor_soort|ev|neut_pron|aanw__Number=Sing|PronType=Dem": {POS: ADP},
    "Prep_N_Adv|voor_soort|ev|neut_pron|vrag__Number=Sing|PronType=Int": {POS: ADP},
    "Prep_N_Adv|voor_soort|mv|neut_deelv__Gender=Masc|Number=Sing|PronType=Tot": {
        POS: ADP
    },
    "Prep_N_Conj_N__Number=Sing": {POS: ADP},
    "Prep_N_Conj__AdpType=Prep": {POS: ADP},
    "Prep_N_Prep_N__Number=Sing": {POS: ADP},
    "Prep_N_Prep|voor_soort|ev|dat_voor__Number=Sing": {POS: ADP},
    "Prep_N_Prep|voor_soort|ev|neut_voor__AdpType=Prep": {POS: ADP},
    "Prep_N_Prep|voor_soort|ev|neut_voor__Number=Sing": {POS: ADP},
    "Prep_N_Prep|voor_soort|mv|neut_voor__Number=Plur": {POS: ADP},
    "Prep_N_V__Case=Nom|Number=Sing|Tense=Past|VerbForm=Part": {POS: ADP},
    "Prep_Num_N__Definite=Def|Number=Sing": {POS: ADP},
    "Prep_Num__Case=Nom|Degree=Sup|PronType=Ind": {POS: ADP},
    "Prep_Num__Degree=Cmp|PronType=Ind": {POS: ADP},
    "Prep_N|voor_eigen|ev|neut__Number=Sing": {POS: ADP},
    "Prep_N|voor_soort|ev|dat__AdpType=Prep": {POS: ADP},
    "Prep_N|voor_soort|ev|dat__Case=Dat|Number=Sing": {POS: ADP},
    "Prep_N|voor_soort|ev|neut__AdpType=Prep": {POS: ADP},
    "Prep_N|voor_soort|ev|neut__Gender=Masc|Number=Sing|PronType=Tot": {POS: ADP},
    "Prep_N|voor_soort|ev|neut__Number=Sing": {POS: ADP},
    "Prep_N|voor_soort|mv|neut__AdpType=Prep": {POS: ADP},
    "Prep_N|voor_soort|mv|neut__Number=Plur": {POS: ADP},
    "Prep_Prep_Adj|voor_voor_adv|stell|onverv__Gender=Masc|Number=Sing|PronType=Tot": {
        POS: ADP
    },
    "Prep_Prep_Adv__Degree=Pos": {POS: ADP},
    "Prep_Pron_Adj__Degree=Cmp|Number=Sing|Person=3": {POS: ADP},
    "Prep_Pron_N_Adv__Number=Plur": {POS: ADP},
    "Prep_Pron_N__AdpType=Prep": {POS: ADP},
    "Prep_Pron_N__Case=Dat|Number=Sing": {POS: ADP},
    "Prep_Pron|voor_aanw|neut|zelfst___": {POS: ADP},
    "Prep_Pron|voor_onbep|neut|attr___": {POS: ADP},
    "Prep_Pron|voor_onbep|neut|zelfst___": {POS: ADP},
    "Prep_Pron|voor_rec|neut__AdpType=Prep": {POS: ADP},
    "Prep_Pron|voor_rec|neut___": {POS: ADP},
    "Prep_Pron|voor_ref|3|evofmv__Number=Plur,Sing|Person=3": {POS: ADP},
    "Prep_Punc_N_Conj_N__AdpType=Prep": {POS: ADP},
    "Prep_V_N__Number=Sing|Tense=Pres|VerbForm=Part": {POS: ADP},
    "Prep_V_Pron_Pron_Adv__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|PronType=Dem|Tense=Pres|VerbForm=Fin": {
        POS: ADP
    },
    "Prep_V|voor_intrans|inf__VerbForm=Inf": {POS: ADP},
    "Prep_V|voorinf_trans|inf__VerbForm=Inf": {POS: ADP},
    "Prep|achter__AdpType=Post": {POS: ADP},
    "Prep|comb__AdpType=Circ": {POS: ADP},
    "Prep|voor__AdpType=Prep": {POS: ADP},
    "Prep|voorinf__AdpType=Prep|PartType=Inf": {POS: ADP},
    "Pron_Adj_N_Punc_Art_Adj_N_Prep_Art_Adj_N__NumType=Card": {POS: PRON},
    "Pron_Adj__Case=Nom|Degree=Sup|Number=Sing|Person=2|Poss=Yes|PronType=Prs": {
        POS: PRON
    },
    "Pron_Adj__Degree=Cmp|PronType=Ind": {POS: PRON},
    "Pron_Adv|vrag|neut|attr_deelv__PronType=Int": {POS: PRON},
    "Pron_Art_N_N__Number=Plur|PronType=Ind": {POS: PRON},
    "Pron_Art__Number=Sing|PronType=Int": {POS: PRON},
    "Pron_N_Adv__Number=Sing|PronType=Ind": {POS: PRON},
    "Pron_N_V_Adv_Num_Punc__Aspect=Imp|Definite=Def|Mood=Ind|Number=Sing|Person=3|PronType=Ind|Tense=Pres|VerbForm=Fin": {
        POS: PRON
    },
    "Pron_N_V_Conj_N__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|PronType=Ind|Tense=Pres|VerbForm=Fin": {
        POS: PRON
    },
    "Pron_N__Case=Gen|Number=Sing|PronType=Ind": {POS: PRON},
    "Pron_N__Number=Sing|PronType=Ind": {POS: PRON},
    "Pron_N|aanw|gen|attr_soort|mv|neut__Case=Gen|Number=Plur|PronType=Dem": {
        POS: PRON
    },
    "Pron_N|onbep|neut|attr_soort|ev|neut__Number=Sing|PronType=Ind": {POS: PRON},
    "Pron_Prep_Art__Number=Sing|PronType=Int": {POS: PRON},
    "Pron_Prep_Art__Number=Sing|PronType=Rel": {POS: PRON},
    "Pron_Prep_N__Number=Plur|PronType=Int": {POS: PRON},
    "Pron_Prep|betr|neut|zelfst_voor__PronType=Rel": {POS: PRON},
    "Pron_Prep|onbep|neut|zelfst_voor__PronType=Ind": {POS: PRON},
    "Pron_Prep|vrag|neut|attr_voor__PronType=Int": {POS: PRON},
    "Pron_Pron_V__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|PronType=Rel|Tense=Pres|VerbForm=Fin": {
        POS: PRON
    },
    "Pron_Pron__Person=3|PronType=Prs|Reflex=Yes": {POS: PRON},
    "Pron_V_V__Aspect=Imp|Mood=Ind|Person=3|PronType=Dem|Tense=Pres|VerbForm=Inf": {
        POS: PRON
    },
    "Pron_V__Case=Gen|Number=Sing|Person=3|Poss=Yes|PronType=Prs|VerbForm=Inf": {
        POS: PRON
    },
    "Pron_V__Number=Plur|Person=1|Poss=Yes|PronType=Prs|VerbForm=Inf": {POS: PRON},
    "Pron|aanw|dat|attr__Case=Dat|PronType=Dem": {POS: PRON},
    "Pron|aanw|gen|attr__Case=Gen|PronType=Dem": {POS: PRON},
    "Pron|aanw|neut|attr__PronType=Dem": {POS: PRON},
    "Pron|aanw|neut|attr|weigen__PronType=Dem": {POS: PRON},
    "Pron|aanw|neut|attr|wzelf__PronType=Dem": {POS: PRON},
    "Pron|aanw|neut|zelfst__PronType=Dem": {POS: PRON},
    "Pron|betr|gen|zelfst__Case=Gen|PronType=Rel": {POS: PRON},
    "Pron|betr|neut|attr__PronType=Rel": {POS: PRON},
    "Pron|betr|neut|zelfst__PronType=Rel": {POS: PRON},
    "Pron|bez|1|ev|neut|attr__Number=Sing|Person=1|Poss=Yes|PronType=Prs": {POS: PRON},
    "Pron|bez|1|mv|neut|attr__Number=Plur|Person=1|Poss=Yes|PronType=Prs": {POS: PRON},
    "Pron|bez|2|ev|neut|attr__Number=Sing|Person=2|Poss=Yes|PronType=Prs": {POS: PRON},
    "Pron|bez|2|mv|neut|attr__Number=Plur|Person=2|Poss=Yes|PronType=Prs": {POS: PRON},
    "Pron|bez|3|ev|gen|attr__Case=Gen|Number=Sing|Person=3|Poss=Yes|PronType=Prs": {
        POS: PRON
    },
    "Pron|bez|3|ev|neut|attr__Number=Sing|Person=3|Poss=Yes|PronType=Prs": {POS: PRON},
    "Pron|bez|3|ev|neut|zelfst__Number=Sing|Person=3|Poss=Yes|PronType=Prs": {
        POS: PRON
    },
    "Pron|bez|3|mv|neut|attr__Number=Plur|Person=3|Poss=Yes|PronType=Prs": {POS: PRON},
    "Pron|onbep|gen|attr__Case=Gen|PronType=Ind": {POS: PRON},
    "Pron|onbep|gen|zelfst__Case=Gen|PronType=Ind": {POS: PRON},
    "Pron|onbep|neut|attr__PronType=Ind": {POS: PRON},
    "Pron|onbep|neut|zelfst__PronType=Ind": {POS: PRON},
    "Pron|per|1|ev|datofacc__Case=Acc,Dat|Number=Sing|Person=1|PronType=Prs": {
        POS: PRON
    },
    "Pron|per|1|ev|nom__Case=Nom|Number=Sing|Person=1|PronType=Prs": {POS: PRON},
    "Pron|per|1|mv|datofacc__Case=Acc,Dat|Number=Plur|Person=1|PronType=Prs": {
        POS: PRON
    },
    "Pron|per|1|mv|nom__Case=Nom|Number=Plur|Person=1|PronType=Prs": {POS: PRON},
    "Pron|per|2|ev|datofacc__Case=Acc,Dat|Number=Sing|Person=2|PronType=Prs": {
        POS: PRON
    },
    "Pron|per|2|ev|nom__Case=Nom|Number=Sing|Person=2|PronType=Prs": {POS: PRON},
    "Pron|per|2|mv|datofacc__Case=Acc,Dat|Number=Plur|Person=2|PronType=Prs": {
        POS: PRON
    },
    "Pron|per|2|mv|nom__Case=Nom|Number=Plur|Person=2|PronType=Prs": {POS: PRON},
    "Pron|per|3|evofmv|datofacc__Case=Acc,Dat|Number=Plur,Sing|Person=3|PronType=Prs": {
        POS: PRON
    },
    "Pron|per|3|evofmv|nom__Case=Nom|Number=Plur,Sing|Person=3|PronType=Prs": {
        POS: PRON
    },
    "Pron|per|3|ev|datofacc__Case=Acc,Dat|Number=Sing|Person=3|PronType=Prs": {
        POS: PRON
    },
    "Pron|per|3|ev|nom__Case=Nom|Number=Sing|Person=3|PronType=Prs": {POS: PRON},
    "Pron|per|3|mv|datofacc__Case=Acc,Dat|Number=Plur|Person=3|PronType=Prs": {
        POS: PRON
    },
    "Pron|rec|gen__Case=Gen|PronType=Rcp": {POS: PRON},
    "Pron|rec|neut__PronType=Rcp": {POS: PRON},
    "Pron|ref|1|ev__Number=Sing|Person=1|PronType=Prs|Reflex=Yes": {POS: PRON},
    "Pron|ref|1|mv__Number=Plur|Person=1|PronType=Prs|Reflex=Yes": {POS: PRON},
    "Pron|ref|2|ev__Number=Sing|Person=2|PronType=Prs|Reflex=Yes": {POS: PRON},
    "Pron|ref|3|evofmv__Number=Plur,Sing|Person=3|PronType=Prs|Reflex=Yes": {POS: PRON},
    "Pron|vrag|neut|attr__PronType=Int": {POS: PRON},
    "Pron|vrag|neut|zelfst__PronType=Int": {POS: PRON},
    "Punc_Int_Punc_N_N_N_Punc_Pron_V_Pron_Adj_V_Punc___": {POS: PUNCT},
    "Punc_N_Punc_N___": {POS: PUNCT},
    "Punc_Num_Num___": {POS: PUNCT},
    "Punc_Num___": {POS: PUNCT},
    "Punc|aanhaaldubb__PunctType=Quot": {POS: PUNCT},
    "Punc|aanhaalenk__PunctType=Quot": {POS: PUNCT},
    "Punc|dubbpunt__PunctType=Colo": {POS: PUNCT},
    "Punc|haakopen__PunctSide=Ini|PunctType=Brck": {POS: PUNCT},
    "Punc|haaksluit__PunctSide=Fin|PunctType=Brck": {POS: PUNCT},
    "Punc|hellip__PunctType=Peri": {POS: PUNCT},
    "Punc|isgelijk___": {POS: PUNCT},
    "Punc|komma__PunctType=Comm": {POS: PUNCT},
    "Punc|liggstreep___": {POS: PUNCT},
    "Punc|maal___": {POS: PUNCT},
    "Punc|punt__PunctType=Peri": {POS: PUNCT},
    "Punc|puntkomma__PunctType=Semi": {POS: PUNCT},
    "Punc|schuinstreep___": {POS: PUNCT},
    "Punc|uitroep__PunctType=Excl": {POS: PUNCT},
    "Punc|vraag__PunctType=Qest": {POS: PUNCT},
    "V_Adv_Art_N_Prep_Pron_N__Degree=Pos|Number=Plur|Person=2|Subcat=Tran": {POS: VERB},
    "V_Adv__Degree=Pos|Subcat=Tran": {POS: VERB},
    "V_Art_N_Num_N__Aspect=Imp|Definite=Def|Mood=Ind|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin|VerbType=Mod": {
        POS: VERB
    },
    "V_Art_N__Number=Sing|Subcat=Tran": {POS: VERB},
    "V_Conj_N_N__Number=Sing|Subcat=Tran|Tense=Past|VerbForm=Part": {POS: VERB},
    "V_Conj_Pron__Subcat=Tran|Tense=Past|VerbForm=Part": {POS: VERB},
    "V_N_Conj_Adj_N_Prep_Art_N__Degree=Pos|Number=Sing|Subcat=Tran|Tense=Past|VerbForm=Part": {
        POS: VERB
    },
    "V_N_N__Number=Sing|Subcat=Intr|Tense=Pres|VerbForm=Part": {POS: VERB},
    "V_N_N__Number=Sing|Subcat=Tran|Tense=Past|VerbForm=Part": {POS: VERB},
    "V_N_V__Aspect=Imp|Mood=Ind|Number=Sing|Subcat=Intr|Tense=Pres|VerbForm=Inf": {
        POS: VERB
    },
    "V_N__Number=Plur|Subcat=Tran|Tense=Past|VerbForm=Part": {POS: VERB},
    "V_N|trans|imp_eigen|ev|neut__Number=Sing|Subcat=Tran": {POS: VERB},
    "V_Prep|intrans|verldw|onverv_voor__Subcat=Intr|Tense=Past|VerbForm=Part": {
        POS: VERB
    },
    "V_Pron_Adv_Adv_Pron_V__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Subcat=Tran|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V_Pron_Adv__Aspect=Imp|Degree=Pos|Mood=Ind|Number=Sing|Person=2|Subcat=Tran|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V_Pron_V__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Subcat=Tran|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V_Pron__VerbType=Aux,Cop": {POS: VERB},
    "V_V|hulp|imp_intrans|inf__VerbForm=Inf|VerbType=Mod": {POS: VERB},
    "V|hulpofkopp|conj__Mood=Sub|VerbForm=Fin": {POS: VERB},
    "V|hulpofkopp|conj__Mood=Sub|VerbForm=Fin|VerbType=Aux,Cop": {POS: VERB},
    "V|hulpofkopp|imp__Mood=Imp|VerbForm=Fin": {POS: VERB},
    "V|hulpofkopp|imp__Mood=Imp|VerbForm=Fin|VerbType=Aux,Cop": {POS: VERB},
    "V|hulpofkopp|inf__VerbForm=Inf": {POS: VERB},
    "V|hulpofkopp|inf__VerbForm=Inf|VerbType=Aux,Cop": {POS: VERB},
    "V|hulpofkopp|inf|subst__VerbForm=Inf": {POS: VERB},
    "V|hulpofkopp|ott|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulpofkopp|ott|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Tense=Pres|VerbForm=Fin|VerbType=Aux,Cop": {
        POS: VERB
    },
    "V|hulpofkopp|ott|1|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulpofkopp|ott|1|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin|VerbType=Aux,Cop": {
        POS: VERB
    },
    "V|hulpofkopp|ott|2|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulpofkopp|ott|2|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Tense=Pres|VerbForm=Fin|VerbType=Aux,Cop": {
        POS: VERB
    },
    "V|hulpofkopp|ott|3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulpofkopp|ott|3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin|VerbType=Aux,Cop": {
        POS: VERB
    },
    "V|hulpofkopp|ovt|1of2of3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Tense=Past|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulpofkopp|ovt|1of2of3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Tense=Past|VerbForm=Fin|VerbType=Aux,Cop": {
        POS: VERB
    },
    "V|hulpofkopp|ovt|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Tense=Past|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulpofkopp|ovt|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Tense=Past|VerbForm=Fin|VerbType=Aux,Cop": {
        POS: VERB
    },
    "V|hulpofkopp|tegdw|vervneut__Case=Nom|Tense=Pres|VerbForm=Part": {POS: VERB},
    "V|hulpofkopp|tegdw|vervneut__Case=Nom|Tense=Pres|VerbForm=Part|VerbType=Aux,Cop": {
        POS: VERB
    },
    "V|hulpofkopp|verldw|onverv__Tense=Past|VerbForm=Part": {POS: VERB},
    "V|hulpofkopp|verldw|onverv__Tense=Past|VerbForm=Part|VerbType=Aux,Cop": {
        POS: VERB
    },
    "V|hulp|conj__Mood=Sub|VerbForm=Fin|VerbType=Mod": {POS: VERB},
    "V|hulp|inf__VerbForm=Inf": {POS: VERB},
    "V|hulp|inf__VerbForm=Inf|VerbType=Mod": {POS: VERB},
    "V|hulp|ott|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulp|ott|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Tense=Pres|VerbForm=Fin|VerbType=Mod": {
        POS: VERB
    },
    "V|hulp|ott|1|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulp|ott|1|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin|VerbType=Mod": {
        POS: VERB
    },
    "V|hulp|ott|2|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulp|ott|2|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Tense=Pres|VerbForm=Fin|VerbType=Mod": {
        POS: VERB
    },
    "V|hulp|ott|3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulp|ott|3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin|VerbType=Mod": {
        POS: VERB
    },
    "V|hulp|ovt|1of2of3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Tense=Past|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulp|ovt|1of2of3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Tense=Past|VerbForm=Fin|VerbType=Mod": {
        POS: VERB
    },
    "V|hulp|ovt|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Tense=Past|VerbForm=Fin": {
        POS: VERB
    },
    "V|hulp|ovt|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Tense=Past|VerbForm=Fin|VerbType=Mod": {
        POS: VERB
    },
    "V|hulp|verldw|onverv__Tense=Past|VerbForm=Part": {POS: VERB},
    "V|hulp|verldw|onverv__Tense=Past|VerbForm=Part|VerbType=Mod": {POS: VERB},
    "V|intrans|conj__Mood=Sub|Subcat=Intr|VerbForm=Fin": {POS: VERB},
    "V|intrans|imp__Mood=Imp|Subcat=Intr|VerbForm=Fin": {POS: VERB},
    "V|intrans|inf__Subcat=Intr|VerbForm=Inf": {POS: VERB},
    "V|intrans|inf|subst__Subcat=Intr|VerbForm=Inf": {POS: VERB},
    "V|intrans|ott|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Subcat=Intr|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|intrans|ott|1|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Subcat=Intr|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|intrans|ott|2|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Subcat=Intr|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|intrans|ott|3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Subcat=Intr|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|intrans|ovt|1of2of3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Subcat=Intr|Tense=Past|VerbForm=Fin": {
        POS: VERB
    },
    "V|intrans|ovt|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Subcat=Intr|Tense=Past|VerbForm=Fin": {
        POS: VERB
    },
    "V|intrans|tegdw|onverv__Subcat=Intr|Tense=Pres|VerbForm=Part": {POS: VERB},
    "V|intrans|tegdw|vervmv__Number=Plur|Subcat=Intr|Tense=Pres|VerbForm=Part": {
        POS: VERB
    },
    "V|intrans|tegdw|vervneut__Case=Nom|Subcat=Intr|Tense=Pres|VerbForm=Part": {
        POS: VERB
    },
    "V|intrans|tegdw|vervvergr__Degree=Cmp|Subcat=Intr|Tense=Pres|VerbForm=Part": {
        POS: VERB
    },
    "V|intrans|verldw|onverv__Subcat=Intr|Tense=Past|VerbForm=Part": {POS: VERB},
    "V|intrans|verldw|vervmv__Number=Plur|Subcat=Intr|Tense=Past|VerbForm=Part": {
        POS: VERB
    },
    "V|intrans|verldw|vervneut__Case=Nom|Subcat=Intr|Tense=Past|VerbForm=Part": {
        POS: VERB
    },
    "V|refl|imp__Mood=Imp|Reflex=Yes|VerbForm=Fin": {POS: VERB},
    "V|refl|inf__Reflex=Yes|VerbForm=Inf": {POS: VERB},
    "V|refl|inf|subst__Reflex=Yes|VerbForm=Inf": {POS: VERB},
    "V|refl|ott|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Reflex=Yes|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|refl|ott|1|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Reflex=Yes|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|refl|ott|2|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Reflex=Yes|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|refl|ott|3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Reflex=Yes|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|refl|ovt|1of2of3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Reflex=Yes|Tense=Past|VerbForm=Fin": {
        POS: VERB
    },
    "V|refl|ovt|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Reflex=Yes|Tense=Past|VerbForm=Fin": {
        POS: VERB
    },
    "V|refl|tegdw|vervneut__Case=Nom|Reflex=Yes|Tense=Pres|VerbForm=Part": {POS: VERB},
    "V|refl|verldw|onverv__Reflex=Yes|Tense=Past|VerbForm=Part": {POS: VERB},
    "V|trans|conj__Mood=Sub|Subcat=Tran|VerbForm=Fin": {POS: VERB},
    "V|trans|imp__Mood=Imp|Subcat=Tran|VerbForm=Fin": {POS: VERB},
    "V|trans|inf__Subcat=Tran|VerbForm=Inf": {POS: VERB},
    "V|trans|inf|subst__Subcat=Tran|VerbForm=Inf": {POS: VERB},
    "V|trans|ott|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Subcat=Tran|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|trans|ott|1|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Subcat=Tran|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|trans|ott|2|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Subcat=Tran|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|trans|ott|3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Subcat=Tran|Tense=Pres|VerbForm=Fin": {
        POS: VERB
    },
    "V|trans|ovt|1of2of3|ev__Aspect=Imp|Mood=Ind|Number=Sing|Subcat=Tran|Tense=Past|VerbForm=Fin": {
        POS: VERB
    },
    "V|trans|ovt|1of2of3|mv__Aspect=Imp|Mood=Ind|Number=Plur|Subcat=Tran|Tense=Past|VerbForm=Fin": {
        POS: VERB
    },
    "V|trans|tegdw|onverv__Subcat=Tran|Tense=Pres|VerbForm=Part": {POS: VERB},
    "V|trans|tegdw|vervneut__Case=Nom|Subcat=Tran|Tense=Pres|VerbForm=Part": {
        POS: VERB
    },
    "V|trans|verldw|onverv__Subcat=Tran|Tense=Past|VerbForm=Part": {POS: VERB},
    "V|trans|verldw|vervmv__Number=Plur|Subcat=Tran|Tense=Past|VerbForm=Part": {
        POS: VERB
    },
    "V|trans|verldw|vervneut__Case=Nom|Subcat=Tran|Tense=Past|VerbForm=Part": {
        POS: VERB
    },
    "V|trans|verldw|vervvergr__Degree=Cmp|Subcat=Tran|Tense=Past|VerbForm=Part": {
        POS: VERB
    },
    "X__Aspect=Imp|Definite=Def|Mood=Ind|Number=Plur|Person=3|Tense=Pres|VerbForm=Fin|VerbType=Mod": {
        POS: X
    },
    "X__Aspect=Imp|Definite=Def|Mood=Ind|Number=Sing|Person=3|PronType=Ind|Tense=Pres|VerbForm=Fin": {
        POS: X
    },
    "X__Aspect=Imp|Degree=Pos|Mood=Ind|Number=Sing|Person=2|Subcat=Tran|Tense=Pres|VerbForm=Fin": {
        POS: X
    },
    "X__Aspect=Imp|Degree=Pos|Mood=Ind|Number=Sing|Person=2|Tense=Past|VerbForm=Part": {
        POS: X
    },
    "X__Aspect=Imp|Degree=Pos|Mood=Ind|Number=Sing|Person=2|Tense=Pres|VerbForm=Inf": {
        POS: X
    },
    "X__Aspect=Imp|Mood=Ind|Number=Sing|Person=1|Tense=Pres|VerbForm=Fin": {POS: X},
    "X__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|PronType=Dem|Tense=Pres|VerbForm=Fin": {
        POS: X
    },
    "X__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|PronType=Rel|Tense=Pres|VerbForm=Fin": {
        POS: X
    },
    "X__Aspect=Imp|Mood=Ind|Number=Sing|Person=2|Subcat=Tran|Tense=Pres|VerbForm=Fin": {
        POS: X
    },
    "X__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|PronType=Ind|Tense=Pres|VerbForm=Fin": {
        POS: X
    },
    "X__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Subcat=Tran|Tense=Pres|VerbForm=Fin": {
        POS: X
    },
    "X__Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin": {POS: X},
    "X__Aspect=Imp|Mood=Ind|Number=Sing|Subcat=Intr|Tense=Pres|VerbForm=Inf": {POS: X},
    "X__Aspect=Imp|Mood=Ind|Number=Sing|Tense=Past|VerbForm=Fin": {POS: X},
    "X__Aspect=Imp|Mood=Ind|Person=3|PronType=Dem|Tense=Pres|VerbForm=Inf": {POS: X},
    "X__Case=Dat|Degree=Pos|Number=Sing": {POS: X},
    "X__Case=Dat|Number=Sing": {POS: X},
    "X__Case=Gen|Definite=Def|Number=Sing": {POS: X},
    "X__Case=Gen|Number=Plur|PronType=Dem": {POS: X},
    "X__Case=Gen|Number=Plur|PronType=Ind": {POS: X},
    "X__Case=Gen|Number=Sing": {POS: X},
    "X__Case=Gen|Number=Sing|Person=3|Poss=Yes|PronType=Prs|VerbForm=Inf": {POS: X},
    "X__Case=Gen|Number=Sing|PronType=Ind": {POS: X},
    "X__Case=Nom|Definite=Def|Degree=Cmp|Gender=Neut": {POS: X},
    "X__Case=Nom|Definite=Def|Degree=Sup": {POS: X},
    "X__Case=Nom|Definite=Def|Degree=Sup|Gender=Neut": {POS: X},
    "X__Case=Nom|Degree=Cmp": {POS: X},
    "X__Case=Nom|Degree=Pos": {POS: X},
    "X__Case=Nom|Degree=Pos|Gender=Neut": {POS: X},
    "X__Case=Nom|Degree=Pos|Number=Plur": {POS: X},
    "X__Case=Nom|Degree=Pos|Number=Sing": {POS: X},
    "X__Case=Nom|Degree=Sup": {POS: X},
    "X__Case=Nom|Degree=Sup|Number=Sing|Person=2|Poss=Yes|PronType=Prs": {POS: X},
    "X__Case=Nom|Degree=Sup|PronType=Ind": {POS: X},
    "X__Case=Nom|Number=Sing|Tense=Past|VerbForm=Part": {POS: X},
    "X__Definite=Def": {POS: X},
    "X__Definite=Def|Degree=Cmp|Gender=Neut": {POS: X},
    "X__Definite=Def|Degree=Pos": {POS: X},
    "X__Definite=Def|Degree=Pos|Number=Sing": {POS: X},
    "X__Definite=Def|Degree=Pos|Variant=Short": {POS: X},
    "X__Definite=Def|Degree=Sup|Gender=Neut": {POS: X},
    "X__Definite=Def|Degree=Sup|Gender=Neut|Number=Sing": {POS: X},
    "X__Definite=Def|Degree=Sup|Gender=Neut|PronType=Ind": {POS: X},
    "X__Definite=Def|Gender=Neut": {POS: X},
    "X__Definite=Def|Gender=Neut|Number=Plur|Person=3": {POS: X},
    "X__Definite=Def|Gender=Neut|Number=Sing": {POS: X},
    "X__Definite=Def|Number=Plur": {POS: X},
    "X__Definite=Def|Number=Sing": {POS: X},
    "X__Definite=Def|Number=Sing|Person=1": {POS: X},
    "X__Definite=Def|Number=Sing|Tense=Past|VerbForm=Part": {POS: X},
    "X__Definite=Def|Number=Sing|Tense=Pres|VerbForm=Part": {POS: X},
    "X__Degree=Cmp": {POS: X},
    "X__Degree=Cmp|Gender=Neut": {POS: X},
    "X__Degree=Cmp|Number=Sing|Person=3": {POS: X},
    "X__Degree=Cmp|PronType=Ind": {POS: X},
    "X__Degree=Cmp|Variant=Short": {POS: X},
    "X__Degree=Pos": {POS: X},
    "X__Degree=Pos|Gender=Neut|Number=Sing": {POS: X},
    "X__Degree=Pos|Mood=Imp|Variant=Short|VerbForm=Fin": {POS: X},
    "X__Degree=Pos|Mood=Sub|VerbForm=Fin": {POS: X},
    "X__Degree=Pos|Number=Plur": {POS: X},
    "X__Degree=Pos|Number=Plur|Person=2|Subcat=Tran": {POS: X},
    "X__Degree=Pos|Number=Plur|Variant=Short": {POS: X},
    "X__Degree=Pos|Number=Sing": {POS: X},
    "X__Degree=Pos|Number=Sing|Person=1|Poss=Yes|PronType=Prs": {POS: X},
    "X__Degree=Pos|Number=Sing|Person=2": {POS: X},
    "X__Degree=Pos|Number=Sing|Person=3": {POS: X},
    "X__Degree=Pos|Number=Sing|PronType=Ind": {POS: X},
    "X__Degree=Pos|Number=Sing|Subcat=Tran|Tense=Past|VerbForm=Part": {POS: X},
    "X__Degree=Pos|Number=Sing|Tense=Past|VerbForm=Part": {POS: X},
    "X__Degree=Pos|Number=Sing|Variant=Short": {POS: X},
    "X__Degree=Pos|PronType=Dem": {POS: X},
    "X__Degree=Pos|Subcat=Tran": {POS: X},
    "X__Degree=Pos|Variant=Short": {POS: X},
    "X__Degree=Pos|Variant=Short|VerbForm=Inf": {POS: X},
    "X__Degree=Pos|VerbForm=Inf": {POS: X},
    "X__Gender=Com|Number=Sing": {POS: X},
    "X__Gender=Neut": {POS: X},
    "X__Gender=Neut|Number=Sing": {POS: X},
    "X__Gender=Neut|VerbForm=Inf": {POS: X},
    "X__Mood=Sub|Number=Sing|VerbForm=Fin": {POS: X},
    "X__Mood=Sub|VerbForm=Fin": {POS: X},
    "X__Number=Plur": {POS: X},
    "X__Number=Plur,Sing|Person=3": {POS: X},
    "X__Number=Plur|Person=1|Poss=Yes|PronType=Prs|VerbForm=Inf": {POS: X},
    "X__Number=Plur|PronType=Ind": {POS: X},
    "X__Number=Plur|PronType=Int": {POS: X},
    "X__Number=Plur|Subcat=Tran|Tense=Past|VerbForm=Part": {POS: X},
    "X__Number=Plur|Tense=Past|VerbForm=Part": {POS: X},
    "X__Number=Sing": {POS: X},
    "X__Number=Sing|Person=3": {POS: X},
    "X__Number=Sing|PronType=Dem": {POS: X},
    "X__Number=Sing|PronType=Ind": {POS: X},
    "X__Number=Sing|PronType=Int": {POS: X},
    "X__Number=Sing|PronType=Rel": {POS: X},
    "X__Number=Sing|Subcat=Intr|Tense=Pres|VerbForm=Part": {POS: X},
    "X__Number=Sing|Subcat=Tran": {POS: X},
    "X__Number=Sing|Subcat=Tran|Tense=Past|VerbForm=Part": {POS: X},
    "X__Number=Sing|Tense=Past|VerbForm=Part": {POS: X},
    "X__Number=Sing|Tense=Pres|VerbForm=Part": {POS: X},
    "X__Person=3|PronType=Prs|Reflex=Yes": {POS: X},
    "X__PronType=Dem": {POS: X},
    "X__PronType=Ind": {POS: X},
    "X__PronType=Int": {POS: X},
    "X__PronType=Rel": {POS: X},
    "X__Subcat=Intr|Tense=Past|VerbForm=Part": {POS: X},
    "X__Subcat=Tran|Tense=Past|VerbForm=Part": {POS: X},
    "X__VerbForm=Inf": {POS: X},
    "X__VerbForm=Inf|VerbType=Mod": {POS: X},
    "X__VerbType=Aux,Cop": {POS: X},
    "X___": {POS: X},
    "_SP": {POS: SPACE},
}



# coding: utf-8
from __future__ import unicode_literals

from ...util import get_doc


def test_de_parser_noun_chunks_standard_de(de_tokenizer):
    text = "Eine Tasse steht auf dem Tisch."
    heads = [1, 1, 0, -1, 1, -2, -4]
    tags = ["ART", "NN", "VVFIN", "APPR", "ART", "NN", "$."]
    deps = ["nk", "sb", "ROOT", "mo", "nk", "nk", "punct"]
    tokens = de_tokenizer(text)
    doc = get_doc(
        tokens.vocab, words=[t.text for t in tokens], tags=tags, deps=deps, heads=heads
    )
    chunks = list(doc.noun_chunks)
    assert len(chunks) == 2
    assert chunks[0].text_with_ws == "Eine Tasse "
    assert chunks[1].text_with_ws == "dem Tisch "


def test_de_extended_chunk(de_tokenizer):
    text = "Die Sängerin singt mit einer Tasse Kaffee Arien."
    heads = [1, 1, 0, -1, 1, -2, -1, -5, -6]
    tags = ["ART", "NN", "VVFIN", "APPR", "ART", "NN", "NN", "NN", "$."]
    deps = ["nk", "sb", "ROOT", "mo", "nk", "nk", "nk", "oa", "punct"]
    tokens = de_tokenizer(text)
    doc = get_doc(
        tokens.vocab, words=[t.text for t in tokens], tags=tags, deps=deps, heads=heads
    )
    chunks = list(doc.noun_chunks)
    assert len(chunks) == 3
    assert chunks[0].text_with_ws == "Die Sängerin "
    assert chunks[1].text_with_ws == "einer Tasse Kaffee "
    assert chunks[2].text_with_ws == "Arien "



# coding: utf8
from __future__ import unicode_literals

import pytest
from thinc.neural.optimizers import Adam
from thinc.neural.ops import NumpyOps
from spacy.attrs import NORM
from spacy.gold import GoldParse
from spacy.vocab import Vocab
from spacy.tokens import Doc
from spacy.pipeline import DependencyParser


@pytest.fixture
def vocab():
    return Vocab(lex_attr_getters={NORM: lambda s: s})


@pytest.fixture
def parser(vocab):
    parser = DependencyParser(vocab)
    parser.cfg["token_vector_width"] = 4
    parser.cfg["hidden_width"] = 32
    # parser.add_label('right')
    parser.add_label("left")
    parser.begin_training([], **parser.cfg)
    sgd = Adam(NumpyOps(), 0.001)

    for i in range(10):
        losses = {}
        doc = Doc(vocab, words=["a", "b", "c", "d"])
        gold = GoldParse(doc, heads=[1, 1, 3, 3], deps=["left", "ROOT", "left", "ROOT"])
        parser.update([doc], [gold], sgd=sgd, losses=losses)
    return parser


def test_no_sentences(parser):
    doc = Doc(parser.vocab, words=["a", "b", "c", "d"])
    doc = parser(doc)
    assert len(list(doc.sents)) >= 1


def test_sents_1(parser):
    doc = Doc(parser.vocab, words=["a", "b", "c", "d"])
    doc[2].sent_start = True
    doc = parser(doc)
    assert len(list(doc.sents)) >= 2
    doc = Doc(parser.vocab, words=["a", "b", "c", "d"])
    doc[1].sent_start = False
    doc[2].sent_start = True
    doc[3].sent_start = False
    doc = parser(doc)
    assert len(list(doc.sents)) == 2


def test_sents_1_2(parser):
    doc = Doc(parser.vocab, words=["a", "b", "c", "d"])
    doc[1].sent_start = True
    doc[2].sent_start = True
    doc = parser(doc)
    assert len(list(doc.sents)) >= 3


def test_sents_1_3(parser):
    doc = Doc(parser.vocab, words=["a", "b", "c", "d"])
    doc[1].sent_start = True
    doc[3].sent_start = True
    doc = parser(doc)
    assert len(list(doc.sents)) >= 3
    doc = Doc(parser.vocab, words=["a", "b", "c", "d"])
    doc[1].sent_start = True
    doc[2].sent_start = False
    doc[3].sent_start = True
    doc = parser(doc)
    assert len(list(doc.sents)) == 3



# coding: utf8
from __future__ import unicode_literals

from spacy.tokens import DocBin


def test_issue4367():
    """Test that docbin init goes well"""
    DocBin()
    DocBin(attrs=["LEMMA"])
    DocBin(attrs=["LEMMA", "ENT_IOB", "ENT_TYPE"])



# coding: utf8
from __future__ import unicode_literals

cimport numpy as np
from libc.math cimport sqrt

import numpy
import numpy.linalg
import warnings
from thinc.neural.util import get_array_module
from collections import defaultdict

from .doc cimport token_by_start, token_by_end, get_token_attr, _get_lca_matrix
from .token cimport TokenC
from ..structs cimport TokenC, LexemeC
from ..typedefs cimport flags_t, attr_t, hash_t
from ..attrs cimport attr_id_t
from ..parts_of_speech cimport univ_pos_t
from ..attrs cimport *
from ..lexeme cimport Lexeme
from ..symbols cimport dep

from ..util import normalize_slice
from ..compat import is_config, basestring_
from ..errors import Errors, TempErrors, Warnings
from .underscore import Underscore, get_ext_args


cdef class Span:
    """A slice from a Doc object.

    DOCS: https://spacy.io/api/span
    """
    @classmethod
    def set_extension(cls, name, **kwargs):
        """Define a custom attribute which becomes available as `Span._`.

        name (unicode): Name of the attribute to set.
        default: Optional default value of the attribute.
        getter (callable): Optional getter function.
        setter (callable): Optional setter function.
        method (callable): Optional method for method extension.
        force (bool): Force overwriting existing attribute.

        DOCS: https://spacy.io/api/span#set_extension
        USAGE: https://spacy.io/usage/processing-pipelines#custom-components-attributes
        """
        if cls.has_extension(name) and not kwargs.get("force", False):
            raise ValueError(Errors.E090.format(name=name, obj="Span"))
        Underscore.span_extensions[name] = get_ext_args(**kwargs)

    @classmethod
    def get_extension(cls, name):
        """Look up a previously registered extension by name.

        name (unicode): Name of the extension.
        RETURNS (tuple): A `(default, method, getter, setter)` tuple.

        DOCS: https://spacy.io/api/span#get_extension
        """
        return Underscore.span_extensions.get(name)

    @classmethod
    def has_extension(cls, name):
        """Check whether an extension has been registered.

        name (unicode): Name of the extension.
        RETURNS (bool): Whether the extension has been registered.

        DOCS: https://spacy.io/api/span#has_extension
        """
        return name in Underscore.span_extensions

    @classmethod
    def remove_extension(cls, name):
        """Remove a previously registered extension.

        name (unicode): Name of the extension.
        RETURNS (tuple): A `(default, method, getter, setter)` tuple of the
            removed extension.

        DOCS: https://spacy.io/api/span#remove_extension
        """
        if not cls.has_extension(name):
            raise ValueError(Errors.E046.format(name=name))
        return Underscore.span_extensions.pop(name)

    def __cinit__(self, Doc doc, int start, int end, label=0, vector=None,
                  vector_norm=None, kb_id=0):
        """Create a `Span` object from the slice `doc[start : end]`.

        doc (Doc): The parent document.
        start (int): The index of the first token of the span.
        end (int): The index of the first token after the span.
        label (uint64): A label to attach to the Span, e.g. for named entities.
        kb_id (uint64): An identifier from a Knowledge Base to capture the meaning of a named entity.
        vector (ndarray[ndim=1, dtype='float32']): A meaning representation
            of the span.
        RETURNS (Span): The newly constructed object.

        DOCS: https://spacy.io/api/span#init
        """
        if not (0 <= start <= end <= len(doc)):
            raise IndexError(Errors.E035.format(start=start, end=end, length=len(doc)))
        self.doc = doc
        self.start = start
        self.start_char = self.doc[start].idx if start < self.doc.length else 0
        self.end = end
        if end >= 1:
            self.end_char = self.doc[end - 1].idx + len(self.doc[end - 1])
        else:
            self.end_char = 0
        if isinstance(label, basestring_):
            label = doc.vocab.strings.add(label)
        if isinstance(kb_id, basestring_):
            kb_id = doc.vocab.strings.add(kb_id)
        if label not in doc.vocab.strings:
            raise ValueError(Errors.E084.format(label=label))
        self.label = label
        self._vector = vector
        self._vector_norm = vector_norm
        self.kb_id = kb_id

    def __richcmp__(self, Span other, int op):
        if other is None:
            if op == 0 or op == 1 or op == 2:
                return False
            else:
                return True
        # <
        if op == 0:
            return self.start_char < other.start_char
        # <=
        elif op == 1:
            return self.start_char <= other.start_char
        # ==
        elif op == 2:
            return (self.doc, self.start_char, self.end_char, self.label, self.kb_id) == (other.doc, other.start_char, other.end_char, other.label, other.kb_id)
        # !=
        elif op == 3:
            return (self.doc, self.start_char, self.end_char, self.label, self.kb_id) != (other.doc, other.start_char, other.end_char, other.label, other.kb_id)
        # >
        elif op == 4:
            return self.start_char > other.start_char
        # >=
        elif op == 5:
            return self.start_char >= other.start_char

    def __hash__(self):
        return hash((self.doc, self.start_char, self.end_char, self.label, self.kb_id))

    def __len__(self):
        """Get the number of tokens in the span.

        RETURNS (int): The number of tokens in the span.

        DOCS: https://spacy.io/api/span#len
        """
        self._recalculate_indices()
        if self.end < self.start:
            return 0
        return self.end - self.start

    def __repr__(self):
        if is_config(python3=True):
            return self.text
        return self.text.encode("utf-8")

    def __getitem__(self, object i):
        """Get a `Token` or a `Span` object

        i (int or tuple): The index of the token within the span, or slice of
            the span to get.
        RETURNS (Token or Span): The token at `span[i]`.

        DOCS: https://spacy.io/api/span#getitem
        """
        self._recalculate_indices()
        if isinstance(i, slice):
            start, end = normalize_slice(len(self), i.start, i.stop, i.step)
            return Span(self.doc, start + self.start, end + self.start)
        else:
            if i < 0:
                return self.doc[self.end + i]
            else:
                return self.doc[self.start + i]

    def __iter__(self):
        """Iterate over `Token` objects.

        YIELDS (Token): A `Token` object.

        DOCS: https://spacy.io/api/span#iter
        """
        self._recalculate_indices()
        for i in range(self.start, self.end):
            yield self.doc[i]

    def __reduce__(self):
        raise NotImplementedError(Errors.E112)

    @property
    def _(self):
        """Custom extension attributes registered via `set_extension`."""
        return Underscore(Underscore.span_extensions, self,
                          start=self.start_char, end=self.end_char)

    def as_doc(self, bint copy_user_data=False):
        """Create a `Doc` object with a copy of the `Span`'s data.

        copy_user_data (bool): Whether or not to copy the original doc's user data.
        RETURNS (Doc): The `Doc` copy of the span.

        DOCS: https://spacy.io/api/span#as_doc
        """
        # TODO: make copy_user_data a keyword-only argument (Python 3 only)
        words = [t.text for t in self]
        spaces = [bool(t.whitespace_) for t in self]
        cdef Doc doc = Doc(self.doc.vocab, words=words, spaces=spaces)
        array_head = [LENGTH, SPACY, LEMMA, ENT_IOB, ENT_TYPE, ENT_ID, ENT_KB_ID]
        if self.doc.is_tagged:
            array_head.append(TAG)
        # If doc parsed add head and dep attribute
        if self.doc.is_parsed:
            array_head.extend([HEAD, DEP])
        # Otherwise add sent_start
        else:
            array_head.append(SENT_START)
        array = self.doc.to_array(array_head)
        array = array[self.start : self.end]
        self._fix_dep_copy(array_head, array)
        doc.from_array(array_head, array)
        doc.noun_chunks_iterator = self.doc.noun_chunks_iterator
        doc.user_hooks = self.doc.user_hooks
        doc.user_span_hooks = self.doc.user_span_hooks
        doc.user_token_hooks = self.doc.user_token_hooks
        doc.vector = self.vector
        doc.vector_norm = self.vector_norm
        doc.tensor = self.doc.tensor[self.start : self.end]
        for key, value in self.doc.cats.items():
            if hasattr(key, "__len__") and len(key) == 3:
                cat_start, cat_end, cat_label = key
                if cat_start == self.start_char and cat_end == self.end_char:
                    doc.cats[cat_label] = value
        if copy_user_data:
            doc.user_data = self.doc.user_data
        return doc

    def _fix_dep_copy(self, attrs, array):
        """ Rewire dependency links to make sure their heads fall into the span
        while still keeping the correct number of sentences. """
        cdef int length = len(array)
        cdef attr_t value
        cdef int i, head_col, ancestor_i
        old_to_new_root = dict()
        if HEAD in attrs:
            head_col = attrs.index(HEAD)
            for i in range(length):
                # if the HEAD refers to a token outside this span, find a more appropriate ancestor
                token = self[i]
                ancestor_i = token.head.i - self.start   # span offset
                if ancestor_i not in range(length):
                    if DEP in attrs:
                        array[i, attrs.index(DEP)] = dep

                    # try finding an ancestor within this span
                    ancestors = token.ancestors
                    for ancestor in ancestors:
                        ancestor_i = ancestor.i - self.start
                        if ancestor_i in range(length):
                            array[i, head_col] = ancestor_i - i

                # if there is no appropriate ancestor, define a new artificial root
                value = array[i, head_col]
                if (i+value) not in range(length):
                    new_root = old_to_new_root.get(ancestor_i, None)
                    if new_root is not None:
                        # take the same artificial root as a previous token from the same sentence
                        array[i, head_col] = new_root - i
                    else:
                        # set this token as the new artificial root
                        array[i, head_col] = 0
                        old_to_new_root[ancestor_i] = i

        return array

    def merge(self, *args, **attributes):
        """Retokenize the document, such that the span is merged into a single
        token.

        **attributes: Attributes to assign to the merged token. By default,
            attributes are inherited from the syntactic root token of the span.
        RETURNS (Token): The newly merged token.
        """
        warnings.warn(Warnings.W013.format(obj="Span"), DeprecationWarning)
        return self.doc.merge(self.start_char, self.end_char, *args,
                              **attributes)

    def get_lca_matrix(self):
        """Calculates a matrix of Lowest Common Ancestors (LCA) for a given
        `Span`, where LCA[i, j] is the index of the lowest common ancestor among
        the tokens span[i] and span[j]. If they have no common ancestor within
        the span, LCA[i, j] will be -1.

        RETURNS (np.array[ndim=2, dtype=numpy.int32]): LCA matrix with shape
            (n, n), where n = len(self).

        DOCS: https://spacy.io/api/span#get_lca_matrix
        """
        return numpy.asarray(_get_lca_matrix(self.doc, self.start, self.end))

    def similarity(self, other):
        """Make a semantic similarity estimate. The default estimate is cosine
        similarity using an average of word vectors.

        other (object): The object to compare with. By default, accepts `Doc`,
            `Span`, `Token` and `Lexeme` objects.
        RETURNS (float): A scalar similarity score. Higher is more similar.

        DOCS: https://spacy.io/api/span#similarity
        """
        if "similarity" in self.doc.user_span_hooks:
            return self.doc.user_span_hooks["similarity"](self, other)
        if len(self) == 1 and hasattr(other, "orth"):
            if self[0].orth == other.orth:
                return 1.0
        elif isinstance(other, (Doc, Span)) and len(self) == len(other):
            similar = True
            for i in range(len(self)):
                if self[i].orth != getattr(other[i], "orth", None):
                    similar = False
                    break
            if similar:
                return 1.0
        if self.vocab.vectors.n_keys == 0:
            warnings.warn(Warnings.W007.format(obj="Span"))
        if self.vector_norm == 0.0 or other.vector_norm == 0.0:
            warnings.warn(Warnings.W008.format(obj="Span"))
            return 0.0
        vector = self.vector
        xp = get_array_module(vector)
        return xp.dot(vector, other.vector) / (self.vector_norm * other.vector_norm)

    cpdef np.ndarray to_array(self, object py_attr_ids):
        """Given a list of M attribute IDs, export the tokens to a numpy
        `ndarray` of shape `(N, M)`, where `N` is the length of the document.
        The values will be 32-bit integers.

        attr_ids (list[int]): A list of attribute ID ints.
        RETURNS (numpy.ndarray[long, ndim=2]): A feature matrix, with one row
            per word, and one column per attribute indicated in the input
            `attr_ids`.
        """
        cdef int i, j
        cdef attr_id_t feature
        cdef np.ndarray[attr_t, ndim=2] output
        # Make an array from the attributes - otherwise our inner loop is Python
        # dict iteration
        cdef np.ndarray[attr_t, ndim=1] attr_ids = numpy.asarray(py_attr_ids, dtype=numpy.uint64)
        cdef int length = self.end - self.start
        output = numpy.ndarray(shape=(length, len(attr_ids)), dtype=numpy.uint64)
        for i in range(self.start, self.end):
            for j, feature in enumerate(attr_ids):
                output[i-self.start, j] = get_token_attr(&self.doc.c[i], feature)
        return output

    cpdef int _recalculate_indices(self) except -1:
        if self.end > self.doc.length \
        or self.doc.c[self.start].idx != self.start_char \
        or (self.doc.c[self.end-1].idx + self.doc.c[self.end-1].lex.length) != self.end_char:
            start = token_by_start(self.doc.c, self.doc.length, self.start_char)
            if self.start == -1:
                raise IndexError(Errors.E036.format(start=self.start_char))
            end = token_by_end(self.doc.c, self.doc.length, self.end_char)
            if end == -1:
                raise IndexError(Errors.E037.format(end=self.end_char))
            self.start = start
            self.end = end + 1

    @property
    def vocab(self):
        """RETURNS (Vocab): The Span's Doc's vocab."""
        return self.doc.vocab

    @property
    def sent(self):
        """RETURNS (Span): The sentence span that the span is a part of."""
        if "sent" in self.doc.user_span_hooks:
            return self.doc.user_span_hooks["sent"](self)
        # This should raise if not parsed / no custom sentence boundaries
        self.doc.sents
        # Use `sent_start` token attribute to find sentence boundaries
        cdef int n = 0
        if self.doc.is_sentenced:
            # Find start of the sentence
            start = self.start
            while self.doc.c[start].sent_start != 1 and start > 0:
                start += -1
            # Find end of the sentence
            end = self.end
            n = 0
            while end < self.doc.length and self.doc.c[end].sent_start != 1:
                end += 1
                n += 1
                if n >= self.doc.length:
                    break
            return self.doc[start:end]

    @property
    def ents(self):
        """The named entities in the span. Returns a tuple of named entity
        `Span` objects, if the entity recognizer has been applied.

        RETURNS (tuple): Entities in the span, one `Span` per entity.

        DOCS: https://spacy.io/api/span#ents
        """
        ents = []
        for ent in self.doc.ents:
            if ent.start >= self.start and ent.end <= self.end:
                ents.append(ent)
        return ents

    @property
    def has_vector(self):
        """A boolean value indicating whether a word vector is associated with
        the object.

        RETURNS (bool): Whether a word vector is associated with the object.

        DOCS: https://spacy.io/api/span#has_vector
        """
        if "has_vector" in self.doc.user_span_hooks:
            return self.doc.user_span_hooks["has_vector"](self)
        elif self.vocab.vectors.data.size > 0:
            return any(token.has_vector for token in self)
        elif self.doc.tensor.size > 0:
            return True
        else:
            return False

    @property
    def vector(self):
        """A real-valued meaning representation. Defaults to an average of the
        token vectors.

        RETURNS (numpy.ndarray[ndim=1, dtype='float32']): A 1D numpy array
            representing the span's semantics.

        DOCS: https://spacy.io/api/span#vector
        """
        if "vector" in self.doc.user_span_hooks:
            return self.doc.user_span_hooks["vector"](self)
        if self._vector is None:
            self._vector = sum(t.vector for t in self) / len(self)
        return self._vector

    @property
    def vector_norm(self):
        """The L2 norm of the span's vector representation.

        RETURNS (float): The L2 norm of the vector representation.

        DOCS: https://spacy.io/api/span#vector_norm
        """
        if "vector_norm" in self.doc.user_span_hooks:
            return self.doc.user_span_hooks["vector"](self)
        vector = self.vector
        xp = get_array_module(vector)
        if self._vector_norm is None:
            total = (vector*vector).sum()
            self._vector_norm = xp.sqrt(total) if total != 0. else 0.
        return self._vector_norm

    @property
    def tensor(self):
        """The span's slice of the doc's tensor.
        
        RETURNS (ndarray[ndim=2, dtype='float32']): A 2D numpy or cupy array
            representing the span's semantics.
        """
        if self.doc.tensor is None:
            return None
        return self.doc.tensor[self.start : self.end]

    @property
    def sentiment(self):
        """RETURNS (float): A scalar value indicating the positivity or
            negativity of the span.
        """
        if "sentiment" in self.doc.user_span_hooks:
            return self.doc.user_span_hooks["sentiment"](self)
        else:
            return sum([token.sentiment for token in self]) / len(self)

    @property
    def text(self):
        """RETURNS (unicode): The original verbatim text of the span."""
        text = self.text_with_ws
        if self[-1].whitespace_:
            text = text[:-1]
        return text

    @property
    def text_with_ws(self):
        """The text content of the span with a trailing whitespace character if
        the last token has one.

        RETURNS (unicode): The text content of the span (with trailing
            whitespace).
        """
        return "".join([t.text_with_ws for t in self])

    @property
    def noun_chunks(self):
        """Yields base noun-phrase `Span` objects, if the document has been
        syntactically parsed. A base noun phrase, or "NP chunk", is a noun
        phrase that does not permit other NPs to be nested within it – so no
        NP-level coordination, no prepositional phrases, and no relative
        clauses.

        YIELDS (Span): Base noun-phrase `Span` objects.

        DOCS: https://spacy.io/api/span#noun_chunks
        """
        if not self.doc.is_parsed:
            raise ValueError(Errors.E029)
        # Accumulate the result before beginning to iterate over it. This
        # prevents the tokenisation from being changed out from under us
        # during the iteration. The tricky thing here is that Span accepts
        # its tokenisation changing, so it's okay once we have the Span
        # objects. See Issue #375
        spans = []
        cdef attr_t label
        if self.doc.noun_chunks_iterator is not None:
            for start, end, label in self.doc.noun_chunks_iterator(self):
                spans.append(Span(self.doc, start, end, label=label))
        for span in spans:
            yield span

    @property
    def root(self):
        """The token with the shortest path to the root of the
        sentence (or the root itself). If multiple tokens are equally
        high in the tree, the first token is taken.

        RETURNS (Token): The root token.

        DOCS: https://spacy.io/api/span#root
        """
        self._recalculate_indices()
        if "root" in self.doc.user_span_hooks:
            return self.doc.user_span_hooks["root"](self)
        # This should probably be called 'head', and the other one called
        # 'gov'. But we went with 'head' elsewhere, and now we're stuck =/
        cdef int i
        # First, we scan through the Span, and check whether there's a word
        # with head==0, i.e. a sentence root. If so, we can return it. The
        # longer the span, the more likely it contains a sentence root, and
        # in this case we return in linear time.
        for i in range(self.start, self.end):
            if self.doc.c[i].head == 0:
                return self.doc[i]
        # If we don't have a sentence root, we do something that's not so
        # algorithmically clever, but I think should be quite fast,
        # especially for short spans.
        # For each word, we count the path length, and arg min this measure.
        # We could use better tree logic to save steps here...But I
        # think this should be okay.
        cdef int current_best = self.doc.length
        cdef int root = -1
        for i in range(self.start, self.end):
            if self.start <= (i+self.doc.c[i].head) < self.end:
                continue
            words_to_root = _count_words_to_root(&self.doc.c[i], self.doc.length)
            if words_to_root < current_best:
                current_best = words_to_root
                root = i
        if root == -1:
            return self.doc[self.start]
        else:
            return self.doc[root]

    def char_span(self, int start_idx, int end_idx, label=0, kb_id=0, vector=None):
        """Create a `Span` object from the slice `span.text[start : end]`.

        start (int): The index of the first character of the span.
        end (int): The index of the first character after the span.
        label (uint64 or string): A label to attach to the Span, e.g. for
            named entities.
        kb_id (uint64 or string):  An ID from a KB to capture the meaning of a named entity.
        vector (ndarray[ndim=1, dtype='float32']): A meaning representation of
            the span.
        RETURNS (Span): The newly constructed object.
        """
        start_idx += self.start_char
        end_idx += self.start_char
        return self.doc.char_span(start_idx, end_idx)

    @property
    def conjuncts(self):
        """Tokens that are conjoined to the span's root.

        RETURNS (tuple): A tuple of Token objects.

        DOCS: https://spacy.io/api/span#lefts
        """
        return self.root.conjuncts

    @property
    def lefts(self):
        """Tokens that are to the left of the span, whose head is within the
        `Span`.

        YIELDS (Token):A left-child of a token of the span.

        DOCS: https://spacy.io/api/span#lefts
        """
        for token in reversed(self):  # Reverse, so we get tokens in order
            for left in token.lefts:
                if left.i < self.start:
                    yield left

    @property
    def rights(self):
        """Tokens that are to the right of the Span, whose head is within the
        `Span`.

        YIELDS (Token): A right-child of a token of the span.

        DOCS: https://spacy.io/api/span#rights
        """
        for token in self:
            for right in token.rights:
                if right.i >= self.end:
                    yield right

    @property
    def n_lefts(self):
        """The number of tokens that are to the left of the span, whose
        heads are within the span.

        RETURNS (int): The number of leftward immediate children of the
            span, in the syntactic dependency parse.

        DOCS: https://spacy.io/api/span#n_lefts
        """
        return len(list(self.lefts))

    @property
    def n_rights(self):
        """The number of tokens that are to the right of the span, whose
        heads are within the span.

        RETURNS (int): The number of rightward immediate children of the
            span, in the syntactic dependency parse.

        DOCS: https://spacy.io/api/span#n_rights
        """
        return len(list(self.rights))

    @property
    def subtree(self):
        """Tokens within the span and tokens which descend from them.

        YIELDS (Token): A token within the span, or a descendant from it.

        DOCS: https://spacy.io/api/span#subtree
        """
        for word in self.lefts:
            yield from word.subtree
        yield from self
        for word in self.rights:
            yield from word.subtree

    property ent_id:
        """RETURNS (uint64): The entity ID."""
        def __get__(self):
            return self.root.ent_id

        def __set__(self, hash_t key):
            raise NotImplementedError(TempErrors.T007.format(attr="ent_id"))

    property ent_id_:
        """RETURNS (unicode): The (string) entity ID."""
        def __get__(self):
            return self.root.ent_id_

        def __set__(self, hash_t key):
            raise NotImplementedError(TempErrors.T007.format(attr="ent_id_"))

    @property
    def orth_(self):
        """Verbatim text content (identical to `Span.text`). Exists mostly for
        consistency with other attributes.

        RETURNS (unicode): The span's text."""
        return self.text

    @property
    def lemma_(self):
        """RETURNS (unicode): The span's lemma."""
        return " ".join([t.lemma_ for t in self]).strip()

    @property
    def upper_(self):
        """Deprecated. Use `Span.text.upper()` instead."""
        return "".join([t.text_with_ws.upper() for t in self]).strip()

    @property
    def lower_(self):
        """Deprecated. Use `Span.text.lower()` instead."""
        return "".join([t.text_with_ws.lower() for t in self]).strip()

    @property
    def string(self):
        """Deprecated: Use `Span.text_with_ws` instead."""
        return "".join([t.text_with_ws for t in self])

    property label_:
        """RETURNS (unicode): The span's label."""
        def __get__(self):
            return self.doc.vocab.strings[self.label]

        def __set__(self, unicode label_):
            if not label_:
                label_ = ''
            raise NotImplementedError(Errors.E129.format(start=self.start, end=self.end, label=label_))

    property kb_id_:
        """RETURNS (unicode): The named entity's KB ID."""
        def __get__(self):
            return self.doc.vocab.strings[self.kb_id]

        def __set__(self, unicode kb_id_):
            if not kb_id_:
                kb_id_ = ''
            current_label = self.label_
            if not current_label:
                current_label = ''
            raise NotImplementedError(Errors.E131.format(start=self.start, end=self.end,
                                                         label=current_label, kb_id=kb_id_))


cdef int _count_words_to_root(const TokenC* token, int sent_length) except -1:
    # Don't allow spaces to be the root, if there are
    # better candidates
    if Lexeme.c_check_flag(token.lex, IS_SPACE) and token.l_kids == 0 and token.r_kids == 0:
        return sent_length-1
    if Lexeme.c_check_flag(token.lex, IS_PUNCT) and token.l_kids == 0 and token.r_kids == 0:
        return sent_length-1
    cdef int n = 0
    while token.head != 0:
        token += token.head
        n += 1
        if n >= sent_length:
            raise RuntimeError(Errors.E039)
    return n



# coding: utf8
from __future__ import unicode_literals

import plac
import requests
import os
import subprocess
import sys
from wasabi import msg

from .link import link
from ..util import get_package_path
from .. import about


@plac.annotations(
    model=("Model to download (shortcut or name)", "positional", None, str),
    direct=("Force direct download of name + version", "flag", "d", bool),
    pip_args=("Additional arguments to be passed to `pip install` on model install"),
)
def download(model, direct=False, *pip_args):
    """
    Download compatible model from default download path using pip. Model
    can be shortcut, model name or, if --direct flag is set, full model name
    with version. For direct downloads, the compatibility check will be skipped.
    """
    if not require_package("spacy") and "--no-deps" not in pip_args:
        msg.warn(
            "Skipping model package dependencies and setting `--no-deps`. "
            "You don't seem to have the spaCy package itself installed "
            "(maybe because you've built from source?), so installing the "
            "model dependencies would cause spaCy to be downloaded, which "
            "probably isn't what you want. If the model package has other "
            "dependencies, you'll have to install them manually."
        )
        pip_args = pip_args + ("--no-deps",)
    dl_tpl = "{m}-{v}/{m}-{v}.tar.gz#egg={m}=={v}"
    if direct:
        components = model.split("-")
        model_name = "".join(components[:-1])
        version = components[-1]
        dl = download_model(dl_tpl.format(m=model_name, v=version), pip_args)
    else:
        shortcuts = get_json(about.__shortcuts__, "available shortcuts")
        model_name = shortcuts.get(model, model)
        compatibility = get_compatibility()
        version = get_version(model_name, compatibility)
        dl = download_model(dl_tpl.format(m=model_name, v=version), pip_args)
        if dl != 0:  # if download subprocess doesn't return 0, exit
            sys.exit(dl)
        msg.good(
            "Download and installation successful",
            "You can now load the model via spacy.load('{}')".format(model_name),
        )
        # Only create symlink if the model is installed via a shortcut like 'en'.
        # There's no real advantage over an additional symlink for en_core_web_sm
        # and if anything, it's more error prone and causes more confusion.
        if model in shortcuts:
            try:
                # Get package path here because link uses
                # pip.get_installed_distributions() to check if model is a
                # package, which fails if model was just installed via
                # subprocess
                package_path = get_package_path(model_name)
                link(model_name, model, force=True, model_path=package_path)
            except:  # noqa: E722
                # Dirty, but since spacy.download and the auto-linking is
                # mostly a convenience wrapper, it's best to show a success
                # message and loading instructions, even if linking fails.
                msg.warn(
                    "Download successful but linking failed",
                    "Creating a shortcut link for '{}' didn't work (maybe you "
                    "don't have admin permissions?), but you can still load "
                    "the model via its full package name: "
                    "nlp = spacy.load('{}')".format(model, model_name),
                )
        # If a model is downloaded and then loaded within the same process, our
        # is_package check currently fails, because pkg_resources.working_set
        # is not refreshed automatically (see #3923). We're trying to work
        # around this here be requiring the package explicitly.
        require_package(model_name)


def require_package(name):
    try:
        import pkg_resources

        pkg_resources.working_set.require(name)
        return True
    except:  # noqa: E722
        return False


def get_json(url, desc):
    r = requests.get(url)
    if r.status_code != 200:
        msg.fail(
            "Server error ({})".format(r.status_code),
            "Couldn't fetch {}. Please find a model for your spaCy "
            "installation (v{}), and download it manually. For more "
            "details, see the documentation: "
            "https://spacy.io/usage/models".format(desc, about.__version__),
            exits=1,
        )
    return r.json()


def get_compatibility():
    version = about.__version__
    version = version.rsplit(".dev", 1)[0]
    comp_table = get_json(about.__compatibility__, "compatibility table")
    comp = comp_table["spacy"]
    if version not in comp:
        msg.fail("No compatible models found for v{} of spaCy".format(version), exits=1)
    return comp[version]


def get_version(model, comp):
    model = model.rsplit(".dev", 1)[0]
    if model not in comp:
        msg.fail(
            "No compatible model found for '{}' "
            "(spaCy v{}).".format(model, about.__version__),
            exits=1,
        )
    return comp[model][0]


def download_model(filename, user_pip_args=None):
    download_url = about.__download_url__ + "/" + filename
    pip_args = ["--no-cache-dir"]
    if user_pip_args:
        pip_args.extend(user_pip_args)
    cmd = [sys.executable, "-m", "pip", "install"] + pip_args + [download_url]
    return subprocess.call(cmd, env=os.environ.copy())



# coding: utf8
from __future__ import unicode_literals

from .doc import Doc
from .token import Token
from .span import Span
from ._serialize import DocBin

__all__ = ["Doc", "Token", "Span", "DocBin"]



# coding: utf8
from __future__ import unicode_literals

import pytest

DOT_TESTS = [
    ("tel.", ["tel", "."]),
    ("0 zł 99 gr", ["0", "zł", "99", "gr"]),
]

HYPHEN_TESTS = [
    ("cztero-", ["cztero-"]),
    ("jedno-", ["jedno-"]),
    ("dwu-", ["dwu-"]),
    ("trzy-", ["trzy-"]),
]


TESTCASES = DOT_TESTS + HYPHEN_TESTS


@pytest.mark.parametrize("text,expected_tokens", TESTCASES)
def test_tokenizer_handles_testcases(pl_tokenizer, text, expected_tokens):
    tokens = pl_tokenizer(text)
    token_list = [token.text for token in tokens if not token.is_space]
    assert expected_tokens == token_list



# coding: utf8
from __future__ import unicode_literals

from collections import defaultdict

from spacy.pipeline import EntityRecognizer

from spacy.lang.en import English
from spacy.tokens import Span


def test_issue4313():
    """ This should not crash or exit with some strange error code """
    beam_width = 16
    beam_density = 0.0001
    nlp = English()
    ner = EntityRecognizer(nlp.vocab)
    ner.add_label("SOME_LABEL")
    ner.begin_training([])
    nlp.add_pipe(ner)

    # add a new label to the doc
    doc = nlp("What do you think about Apple ?")
    assert len(ner.labels) == 1
    assert "SOME_LABEL" in ner.labels
    apple_ent = Span(doc, 5, 6, label="MY_ORG")
    doc.ents = list(doc.ents) + [apple_ent]

    # ensure the beam_parse still works with the new label
    docs = [doc]
    beams = nlp.entity.beam_parse(
        docs, beam_width=beam_width, beam_density=beam_density
    )

    for doc, beam in zip(docs, beams):
        entity_scores = defaultdict(float)
        for score, ents in nlp.entity.moves.get_beam_parses(beam):
            for start, end, label in ents:
                entity_scores[(start, end, label)] += score



import spacy
import time
import re
import plac
import operator
import datetime
from pathlib import Path
import xml.etree.ElementTree as ET

import conll17_ud_eval
from ud_train import write_conllu
from spacy.lang.lex_attrs import word_shape
from spacy.util import get_lang_class

# All languages in spaCy - in UD format (note that Norwegian is 'no' instead of 'nb')
ALL_LANGUAGES = ("af, ar, bg, bn, ca, cs, da, de, el, en, es, et, fa, fi, fr,"
                 "ga, he, hi, hr, hu, id, is, it, ja, kn, ko, lt, lv, mr, no,"
                 "nl, pl, pt, ro, ru, si, sk, sl, sq, sr, sv, ta, te, th, tl,"
                 "tr, tt, uk, ur, vi, zh")

# Non-parsing tasks that will be evaluated (works for default models)
EVAL_NO_PARSE = ['Tokens', 'Words', 'Lemmas', 'Sentences', 'Feats']

# Tasks that will be evaluated if check_parse=True (does not work for default models)
EVAL_PARSE = ['Tokens', 'Words', 'Lemmas', 'Sentences', 'Feats', 'UPOS', 'XPOS', 'AllTags', 'UAS', 'LAS']

# Minimum frequency an error should have to be printed
PRINT_FREQ = 20

# Maximum number of errors printed per category
PRINT_TOTAL = 10

space_re = re.compile("\s+")


def load_model(modelname, add_sentencizer=False):
    """ Load a specific spaCy model """
    loading_start = time.time()
    nlp = spacy.load(modelname)
    if add_sentencizer:
        nlp.add_pipe(nlp.create_pipe('sentencizer'))
    loading_end = time.time()
    loading_time = loading_end - loading_start
    if add_sentencizer:
        return nlp, loading_time, modelname + '_sentencizer'
    return nlp, loading_time, modelname


def load_default_model_sentencizer(lang):
    """ Load a generic spaCy model and add the sentencizer for sentence tokenization"""
    loading_start = time.time()
    lang_class = get_lang_class(lang)
    nlp = lang_class()
    nlp.add_pipe(nlp.create_pipe('sentencizer'))
    loading_end = time.time()
    loading_time = loading_end - loading_start
    return nlp, loading_time, lang + "_default_" + 'sentencizer'


def split_text(text):
    return [space_re.sub(" ", par.strip()) for par in text.split("\n\n")]


def get_freq_tuples(my_list, print_total_threshold):
    """ Turn a list of errors into frequency-sorted tuples thresholded by a certain total number """
    d = {}
    for token in my_list:
        d.setdefault(token, 0)
        d[token] += 1
    return sorted(d.items(), key=operator.itemgetter(1), reverse=True)[:print_total_threshold]


def _contains_blinded_text(stats_xml):
    """ Heuristic to determine whether the treebank has blinded texts or not """
    tree = ET.parse(stats_xml)
    root = tree.getroot()
    total_tokens = int(root.find('size/total/tokens').text)
    unique_forms = int(root.find('forms').get('unique'))

    # assume the corpus is largely blinded when there are less than 1% unique tokens
    return (unique_forms / total_tokens) < 0.01


def fetch_all_treebanks(ud_dir, languages, corpus, best_per_language):
    """" Fetch the txt files for all treebanks for a given set of languages """
    all_treebanks = dict()
    treebank_size = dict()
    for l in languages:
        all_treebanks[l] = []
        treebank_size[l] = 0

    for treebank_dir in ud_dir.iterdir():
        if treebank_dir.is_dir():
            for txt_path in treebank_dir.iterdir():
                if txt_path.name.endswith('-ud-' + corpus + '.txt'):
                    file_lang = txt_path.name.split('_')[0]
                    if file_lang in languages:
                        gold_path = treebank_dir / txt_path.name.replace('.txt', '.conllu')
                        stats_xml = treebank_dir / "stats.xml"
                        # ignore treebanks where the texts are not publicly available
                        if not _contains_blinded_text(stats_xml):
                            if not best_per_language:
                                all_treebanks[file_lang].append(txt_path)
                            # check the tokens in the gold annotation to keep only the biggest treebank per language
                            else:
                                with gold_path.open(mode='r', encoding='utf-8') as gold_file:
                                    gold_ud = conll17_ud_eval.load_conllu(gold_file)
                                    gold_tokens = len(gold_ud.tokens)
                                if treebank_size[file_lang] < gold_tokens:
                                    all_treebanks[file_lang] = [txt_path]
                                    treebank_size[file_lang] = gold_tokens

    return all_treebanks


def run_single_eval(nlp, loading_time, print_name, text_path, gold_ud, tmp_output_path, out_file, print_header,
                    check_parse, print_freq_tasks):
    """" Run an evaluation of a model nlp on a certain specified treebank """
    with text_path.open(mode='r', encoding='utf-8') as f:
        flat_text = f.read()

    # STEP 1: tokenize text
    tokenization_start = time.time()
    texts = split_text(flat_text)
    docs = list(nlp.pipe(texts))
    tokenization_end = time.time()
    tokenization_time = tokenization_end - tokenization_start

    # STEP 2: record stats and timings
    tokens_per_s = int(len(gold_ud.tokens) / tokenization_time)

    print_header_1 = ['date', 'text_path', 'gold_tokens', 'model', 'loading_time', 'tokenization_time', 'tokens_per_s']
    print_string_1 = [str(datetime.date.today()), text_path.name, len(gold_ud.tokens),
                      print_name, "%.2f" % loading_time, "%.2f" % tokenization_time, tokens_per_s]

    # STEP 3: evaluate predicted tokens and features
    with tmp_output_path.open(mode="w", encoding="utf8") as tmp_out_file:
        write_conllu(docs, tmp_out_file)
    with tmp_output_path.open(mode="r", encoding="utf8") as sys_file:
        sys_ud = conll17_ud_eval.load_conllu(sys_file, check_parse=check_parse)
    tmp_output_path.unlink()
    scores = conll17_ud_eval.evaluate(gold_ud, sys_ud, check_parse=check_parse)

    # STEP 4: format the scoring results
    eval_headers = EVAL_PARSE
    if not check_parse:
        eval_headers = EVAL_NO_PARSE

    for score_name in eval_headers:
        score = scores[score_name]
        print_string_1.extend(["%.2f" % score.precision,
                               "%.2f" % score.recall,
                               "%.2f" % score.f1])
        print_string_1.append("-" if score.aligned_accuracy is None else "%.2f" % score.aligned_accuracy)
        print_string_1.append("-" if score.undersegmented is None else "%.4f" % score.under_perc)
        print_string_1.append("-" if score.oversegmented is None else "%.4f" % score.over_perc)

        print_header_1.extend([score_name + '_p', score_name + '_r', score_name + '_F', score_name + '_acc',
                               score_name + '_under', score_name + '_over'])

        if score_name in print_freq_tasks:
            print_header_1.extend([score_name + '_word_under_ex', score_name + '_shape_under_ex',
                                   score_name + '_word_over_ex', score_name + '_shape_over_ex'])

            d_under_words = get_freq_tuples(score.undersegmented, PRINT_TOTAL)
            d_under_shapes = get_freq_tuples([word_shape(x) for x in score.undersegmented], PRINT_TOTAL)
            d_over_words = get_freq_tuples(score.oversegmented, PRINT_TOTAL)
            d_over_shapes = get_freq_tuples([word_shape(x) for x in score.oversegmented], PRINT_TOTAL)

            # saving to CSV with ; seperator so blinding ; in the example output
            print_string_1.append(
                str({k: v for k, v in d_under_words if v > PRINT_FREQ}).replace(";", "*SEMICOLON*"))
            print_string_1.append(
                str({k: v for k, v in d_under_shapes if v > PRINT_FREQ}).replace(";", "*SEMICOLON*"))
            print_string_1.append(
                str({k: v for k, v in d_over_words if v > PRINT_FREQ}).replace(";", "*SEMICOLON*"))
            print_string_1.append(
                str({k: v for k, v in d_over_shapes if v > PRINT_FREQ}).replace(";", "*SEMICOLON*"))

    # STEP 5: print the formatted results to CSV
    if print_header:
        out_file.write(';'.join(map(str, print_header_1)) + '\n')
    out_file.write(';'.join(map(str, print_string_1)) + '\n')


def run_all_evals(models, treebanks, out_file, check_parse, print_freq_tasks):
    """" Run an evaluation for each language with its specified models and treebanks """
    print_header = True

    for tb_lang, treebank_list in treebanks.items():
        print()
        print("Language", tb_lang)
        for text_path in treebank_list:
            print(" Evaluating on", text_path)

            gold_path = text_path.parent / (text_path.stem + '.conllu')
            print("  Gold data from ", gold_path)

            # nested try blocks to ensure the code can continue with the next iteration after a failure
            try:
                with gold_path.open(mode='r', encoding='utf-8') as gold_file:
                    gold_ud = conll17_ud_eval.load_conllu(gold_file)

                for nlp, nlp_loading_time, nlp_name in models[tb_lang]:
                    try:
                        print("   Benchmarking", nlp_name)
                        tmp_output_path = text_path.parent / str('tmp_' + nlp_name + '.conllu')
                        run_single_eval(nlp, nlp_loading_time, nlp_name, text_path, gold_ud, tmp_output_path, out_file,
                                        print_header, check_parse, print_freq_tasks)
                        print_header = False
                    except Exception as e:
                        print("    Ran into trouble: ", str(e))
            except Exception as e:
                print("   Ran into trouble: ", str(e))


@plac.annotations(
    out_path=("Path to output CSV file", "positional", None, Path),
    ud_dir=("Path to Universal Dependencies corpus", "positional", None, Path),
    check_parse=("Set flag to evaluate parsing performance", "flag", "p", bool),
    langs=("Enumeration of languages to evaluate (default: all)", "option", "l", str),
    exclude_trained_models=("Set flag to exclude trained models", "flag", "t", bool),
    exclude_multi=("Set flag to exclude the multi-language model as default baseline", "flag", "m", bool),
    hide_freq=("Set flag to avoid printing out more detailed high-freq tokenization errors", "flag", "f", bool),
    corpus=("Whether to run on train, dev or test", "option", "c", str),
    best_per_language=("Set flag to only keep the largest treebank for each language", "flag", "b", bool)
)
def main(out_path, ud_dir, check_parse=False, langs=ALL_LANGUAGES, exclude_trained_models=False, exclude_multi=False,
         hide_freq=False, corpus='train', best_per_language=False):
    """"
    Assemble all treebanks and models to run evaluations with.
    When setting check_parse to True, the default models will not be evaluated as they don't have parsing functionality
    """
    languages = [lang.strip() for lang in langs.split(",")]

    print_freq_tasks = []
    if not hide_freq:
        print_freq_tasks = ['Tokens']

    # fetching all relevant treebank from the directory
    treebanks = fetch_all_treebanks(ud_dir, languages, corpus, best_per_language)

    print()
    print("Loading all relevant models for", languages)
    models = dict()

    # multi-lang model
    multi = None
    if not exclude_multi and not check_parse:
        multi = load_model('xx_ent_wiki_sm', add_sentencizer=True)

    # initialize all models with the multi-lang model
    for lang in languages:
        models[lang] = [multi] if multi else []
        # add default models if we don't want to evaluate parsing info
        if not check_parse:
            # Norwegian is 'nb' in spaCy but 'no' in the UD corpora
            if lang == 'no':
                models['no'].append(load_default_model_sentencizer('nb'))
            else:
                models[lang].append(load_default_model_sentencizer(lang))

    # language-specific trained models
    if not exclude_trained_models:
        if 'de' in models:
            models['de'].append(load_model('de_core_news_sm'))
            models['de'].append(load_model('de_core_news_md'))
        if 'el' in models:
            models['el'].append(load_model('el_core_news_sm'))
            models['el'].append(load_model('el_core_news_md'))
        if 'en' in models:
            models['en'].append(load_model('en_core_web_sm'))
            models['en'].append(load_model('en_core_web_md'))
            models['en'].append(load_model('en_core_web_lg'))
        if 'es' in models:
            models['es'].append(load_model('es_core_news_sm'))
            models['es'].append(load_model('es_core_news_md'))
        if 'fr' in models:
            models['fr'].append(load_model('fr_core_news_sm'))
            models['fr'].append(load_model('fr_core_news_md'))
        if 'it' in models:
            models['it'].append(load_model('it_core_news_sm'))
        if 'nl' in models:
            models['nl'].append(load_model('nl_core_news_sm'))
        if 'pt' in models:
            models['pt'].append(load_model('pt_core_news_sm'))

    with out_path.open(mode='w', encoding='utf-8') as out_file:
        run_all_evals(models, treebanks, out_file, check_parse, print_freq_tasks)


if __name__ == "__main__":
    plac.call(main)



# coding: utf8
from __future__ import unicode_literals

from ...symbols import LEMMA, PRON_LEMMA


# Used the table of pronouns at https://sv.wiktionary.org/wiki/deras
MORPH_RULES = {
    "PRP": {
        "jag": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Sing",
            "Case": "Nom",
        },
        "mig": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Sing",
            "Case": "Acc",
        },
        "mej": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Sing",
            "Case": "Acc",
        },
        "du": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": "Sing",
            "Case": "Nom",
        },
        "han": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Three",
            "Number": "Sing",
            "Gender": "Masc",
            "Case": "Nom",
        },
        "honom": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Three",
            "Number": "Sing",
            "Gender": "Masc",
            "Case": "Acc",
        },
        "hon": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Three",
            "Number": "Sing",
            "Gender": "Fem",
            "Case": "Nom",
        },
        "henne": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Three",
            "Number": "Sing",
            "Gender": "Fem",
            "Case": "Acc",
        },
        "det": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Three",
            "Number": "Sing",
            "Gender": "Neut",
        },
        "vi": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Plur",
            "Case": "Nom",
        },
        "oss": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Plur",
            "Case": "Acc",
        },
        "ni": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": "Plur",
            "Case": "Nom",
        },
        "er": {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Number": "Plur"},
        "de": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Three",
            "Number": "Plur",
            "Case": "Nom",
        },
        "dom": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Three",
            "Number": "Plur",
            "Case": ("Nom", "Acc"),
        },
        "dem": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Three",
            "Number": "Plur",
            "Case": "Acc",
        },
        "min": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Sing",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "mitt": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Sing",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "mina": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "din": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": "Sing",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "ditt": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": "Sing",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "dina": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "hans": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": ("Sing", "Plur"),
            "Gender": "Masc",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "hennes": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": ("Sing", "Plur"),
            "Gender": "Fem",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "dess": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": ("Sing", "Plur"),
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "vår": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "våran": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "vårt": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "vårat": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "våra": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "One",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "eran": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "ert": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "erat": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "era": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Two",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
        "deras": {
            LEMMA: PRON_LEMMA,
            "PronType": "Prs",
            "Person": "Three",
            "Number": "Plur",
            "Poss": "Yes",
            "Reflex": "Yes",
        },
    },
    "VBZ": {
        "är": {
            "VerbForm": "Fin",
            "Person": ("One", "Two", "Three"),
            "Tense": "Pres",
            "Mood": "Ind",
        }
    },
    "VBP": {"är": {"VerbForm": "Fin", "Tense": "Pres", "Mood": "Ind"}},
    "VBD": {
        "var": {"VerbForm": "Fin", "Tense": "Past", "Number": "Sing"},
        "vart": {"VerbForm": "Fin", "Tense": "Past", "Number": "Plur"},
    },
}



# coding: utf8
from __future__ import unicode_literals


"""
Example sentences to test spaCy and its language models.

>>> from spacy.lang.sr.examples import sentences
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    # Translations from English
    "Apple планира куповину америчког стартапа за $1 милијарду.",
    "Беспилотни аутомобили пребацују одговорност осигурања на произвођаче.",
    "Лондон је велики град у Уједињеном Краљевству.",
    "Где си ти?",
    "Ко је председник Француске?",
    # Serbian common and slang
    "Moj ћале је инжењер!",
    "Новак Ђоковић је најбољи тенисер света.",
    "У Пироту има добрих кафана!",
    "Музеј Николе Тесле се налази у Београду.",
]



from .conllu2json import conllu2json  # noqa: F401
from .iob2json import iob2json  # noqa: F401
from .conll_ner2json import conll_ner2json  # noqa: F401
from .jsonl2json import ner_jsonl2json  # noqa: F401



# coding=utf-8
from __future__ import unicode_literals

from ...symbols import ORTH, LEMMA


_exc = {}


for exc_data in [
    {ORTH: "ডঃ", LEMMA: "ডক্টর"},
    {ORTH: "ডাঃ", LEMMA: "ডাক্তার"},
    {ORTH: "ড.", LEMMA: "ডক্টর"},
    {ORTH: "ডা.", LEMMA: "ডাক্তার"},
    {ORTH: "মোঃ", LEMMA: "মোহাম্মদ"},
    {ORTH: "মো.", LEMMA: "মোহাম্মদ"},
    {ORTH: "সে.", LEMMA: "সেলসিয়াস"},
    {ORTH: "কি.মি.", LEMMA: "কিলোমিটার"},
    {ORTH: "কি.মি", LEMMA: "কিলোমিটার"},
    {ORTH: "সে.মি.", LEMMA: "সেন্টিমিটার"},
    {ORTH: "সে.মি", LEMMA: "সেন্টিমিটার"},
    {ORTH: "মি.লি.", LEMMA: "মিলিলিটার"},
]:
    _exc[exc_data[ORTH]] = [exc_data]


TOKENIZER_EXCEPTIONS = _exc



# coding: utf8
from __future__ import unicode_literals

import plac
from pathlib import Path
from wasabi import Printer
import srsly
import re

from .converters import conllu2json, iob2json, conll_ner2json
from .converters import ner_jsonl2json


# Converters are matched by file extension except for ner/iob, which are
# matched by file extension and content. To add a converter, add a new
# entry to this dict with the file extension mapped to the converter function
# imported from /converters.
CONVERTERS = {
    "conllubio": conllu2json,
    "conllu": conllu2json,
    "conll": conllu2json,
    "ner": conll_ner2json,
    "iob": iob2json,
    "jsonl": ner_jsonl2json,
}

# File types
FILE_TYPES = ("json", "jsonl", "msg")
FILE_TYPES_STDOUT = ("json", "jsonl")


@plac.annotations(
    input_file=("Input file", "positional", None, str),
    output_dir=("Output directory. '-' for stdout.", "positional", None, str),
    file_type=("Type of data to produce: {}".format(FILE_TYPES), "option", "t", str),
    n_sents=("Number of sentences per doc (0 to disable)", "option", "n", int),
    seg_sents=("Segment sentences (for -c ner)", "flag", "s"),
    model=("Model for sentence segmentation (for -s)", "option", "b", str),
    converter=("Converter: {}".format(tuple(CONVERTERS.keys())), "option", "c", str),
    lang=("Language (if tokenizer required)", "option", "l", str),
    morphology=("Enable appending morphology to tags", "flag", "m", bool),
)
def convert(
    input_file,
    output_dir="-",
    file_type="json",
    n_sents=1,
    seg_sents=False,
    model=None,
    morphology=False,
    converter="auto",
    lang=None,
):
    """
    Convert files into JSON format for use with train command and other
    experiment management functions. If no output_dir is specified, the data
    is written to stdout, so you can pipe them forward to a JSON file:
    $ spacy convert some_file.conllu > some_file.json
    """
    no_print = output_dir == "-"
    msg = Printer(no_print=no_print)
    input_path = Path(input_file)
    if file_type not in FILE_TYPES:
        msg.fail(
            "Unknown file type: '{}'".format(file_type),
            "Supported file types: '{}'".format(", ".join(FILE_TYPES)),
            exits=1,
        )
    if file_type not in FILE_TYPES_STDOUT and output_dir == "-":
        # TODO: support msgpack via stdout in srsly?
        msg.fail(
            "Can't write .{} data to stdout.".format(file_type),
            "Please specify an output directory.",
            exits=1,
        )
    if not input_path.exists():
        msg.fail("Input file not found", input_path, exits=1)
    if output_dir != "-" and not Path(output_dir).exists():
        msg.fail("Output directory not found", output_dir, exits=1)
    input_data = input_path.open("r", encoding="utf-8").read()
    if converter == "auto":
        converter = input_path.suffix[1:]
    if converter == "ner" or converter == "iob":
        converter_autodetect = autodetect_ner_format(input_data)
        if converter_autodetect == "ner":
            msg.info("Auto-detected token-per-line NER format")
            converter = converter_autodetect
        elif converter_autodetect == "iob":
            msg.info("Auto-detected sentence-per-line NER format")
            converter = converter_autodetect
        else:
            msg.warn(
                "Can't automatically detect NER format. Conversion may not succeed. See https://spacy.io/api/cli#convert"
            )
    if converter not in CONVERTERS:
        msg.fail("Can't find converter for {}".format(converter), exits=1)
    # Use converter function to convert data
    func = CONVERTERS[converter]
    data = func(
        input_data,
        n_sents=n_sents,
        seg_sents=seg_sents,
        use_morphology=morphology,
        lang=lang,
        model=model,
        no_print=no_print,
    )
    if output_dir != "-":
        # Export data to a file
        suffix = ".{}".format(file_type)
        output_file = Path(output_dir) / Path(input_path.parts[-1]).with_suffix(suffix)
        if file_type == "json":
            srsly.write_json(output_file, data)
        elif file_type == "jsonl":
            srsly.write_jsonl(output_file, data)
        elif file_type == "msg":
            srsly.write_msgpack(output_file, data)
        msg.good(
            "Generated output file ({} documents): {}".format(len(data), output_file)
        )
    else:
        # Print to stdout
        if file_type == "json":
            srsly.write_json("-", data)
        elif file_type == "jsonl":
            srsly.write_jsonl("-", data)


def autodetect_ner_format(input_data):
    # guess format from the first 20 lines
    lines = input_data.split("\n")[:20]
    format_guesses = {"ner": 0, "iob": 0}
    iob_re = re.compile(r"\S+\|(O|[IB]-\S+)")
    ner_re = re.compile(r"\S+\s+(O|[IB]-\S+)$")
    for line in lines:
        line = line.strip()
        if iob_re.search(line):
            format_guesses["iob"] += 1
        if ner_re.search(line):
            format_guesses["ner"] += 1
    if format_guesses["iob"] == 0 and format_guesses["ner"] > 0:
        return "ner"
    if format_guesses["ner"] == 0 and format_guesses["iob"] > 0:
        return "iob"
    return None
